@@ -155,6 +155,7 @@ gnome_font_unsized_closest (const char *family_name,
 
 	if (!fontmap) gff_refresh_fontmap ();
 	g_return_val_if_fail (fontmap != NULL, NULL);
+	g_return_val_if_fail (fontlist != NULL, NULL);
 
 	/* This should be reimplemented to use the gnome_font_family_hash. */
 
@@ -173,7 +174,13 @@ gnome_font_unsized_closest (const char *family_name,
 		}
 	}
 
-	if (!best) return NULL;
+	if (!best) {
+		best = g_hash_table_lookup (fontmap, "Helvetica");
+	}
+
+	if (!best) {
+		best = GNOME_FONT_FACE (g_list_last (fontlist)->data);
+	}
 
 	gnome_font_face_ref (best);
 
@@ -371,16 +378,28 @@ gnome_font_face_is_italic (const GnomeFontFace * face)
 	return face->private->italic;
 }
 
+gboolean
+gnome_font_face_is_fixed_width (const GnomeFontFace * face)
+{
+	g_return_val_if_fail (face != NULL, FALSE);
+	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), FALSE);
+
+	return face->private->fixed_width;
+}
+
+
+
 /* fixme: */
 /* Returns the glyph width in 0.001 unit */
 
-gdouble gnome_font_face_get_glyph_width (const GnomeFontFace * face, gint glyph)
+gdouble
+gnome_font_face_get_glyph_width (const GnomeFontFace * face, gint glyph)
 {
 	g_return_val_if_fail (face != NULL, 0.0);
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), 0.0);
 	g_return_val_if_fail (GFF_METRICS (face), 0.0);
 
-	if ((glyph < 0) && (glyph >= face->private->num_glyphs)) glyph = 0;
+	if ((glyph < 0) || (glyph >= face->private->num_glyphs)) glyph = 0;
 
 	return face->private->glyphs[glyph].advance.x;
 }
@@ -441,7 +460,7 @@ gnome_font_face_get_glyph_ps_name (const GnomeFontFace * face, gint glyph)
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
 	g_return_val_if_fail (GFF_METRICS (face), NULL);
 
-	if ((glyph < 0) && (glyph >= face->private->num_glyphs)) glyph = 0;
+	if ((glyph < 0) || (glyph >= face->private->num_glyphs)) glyph = 0;
 
 	return face->private->glyphs[glyph].psname;
 }
@@ -726,9 +745,7 @@ gff_add_mapping (const char *fontname,
 	face->private->fullname = g_strdup (fullname);
 	face->private->familyname = g_strdup (familyname);
 	face->private->weight = g_strdup (weight);
-#if 0
-	face->private->widths = NULL;
-#endif
+
 	if ( alias )
 		face->private->alias = g_strdup( alias );
 	else
@@ -932,112 +949,6 @@ gnome_font_face_gt1_load (GnomeFontFace * face)
 	return TRUE;
 }
 
-#if 0
-/* Looks up the name, returning the glyph number in the font's
-   standard encoding. Return -1 if the glyph is not found in the
-   font's standard encoding.
-
-   There are two problems with this function: first, the implementation
-   is quite slow (the analogous operation in libhnj uses a _much_
-   faster hash table implementation. Second, we eventually want to
-   break free of the font's standard encoding. My guess is that
-   values < 256 will be glyphnums in the standard encoding, while
-   values >= 256 will be in the "extension" encoding.
-
-   Anyway, the point of this function is merely that it works. */
-static int
-gnome_font_find_glyphnum (Font_Info *fi, const char *name)
-{
-  int i;
-
-  for (i = 0; i < fi->numOfChars; i++)
-    {
-      if (!strcmp (name, fi->cmi[i].name))
-	return fi->cmi[i].code;
-    }
-  return -1;
-}
-
-static int
-gnome_font_find_unicode_encoding (const GnomeFontType1EncTab *enc, int n_enc,
-				  const char *glyphname)
-{
-  int i;
-
-  for (i = 0; i < n_enc; i++)
-    {
-      if (!strcmp (glyphname, enc[i].type1_name))
-	return enc[i].unicode;
-    }
-  return -1;
-}
-
-/* Add a mapping to the font's coverage table. */
-static void
-gnome_font_add_unimapping (GnomeFontMap *fontmap, int unicode, int glyphnum)
-{
-  int page;
-  int **cov_pages;
-  int *cov;
-  int i;
-
-  page = unicode >> 8;
-
-  if (fontmap->private->cov_pages == NULL)
-    {
-      cov_pages = g_new (int *, 1);
-      fontmap->private->cov_pages = cov_pages;
-      fontmap->private->first_cov_page = page;
-      fontmap->private->num_cov_pages = 1;
-      cov = g_new (int, 256);
-      for (i = 0; i < 256; i++)
-	cov[i] = -1;
-      cov_pages[0] = cov;
-    }
-  else if (page < fontmap->private->first_cov_page ||
-	   page >= fontmap->private->first_cov_page + fontmap->private->num_cov_pages)
-    {
-      int first_cov_page, num_cov_pages;
-
-      first_cov_page = MIN (page, fontmap->private->first_cov_page);
-      num_cov_pages = MAX (page + 1 - first_cov_page, fontmap->private->num_cov_pages);
-      cov_pages = g_new (int *, num_cov_pages);
-
-      for (i = 0; i < first_cov_page - fontmap->private->first_cov_page; i++)
-	cov_pages[i] = NULL;
-      memcpy (cov_pages + fontmap->private->first_cov_page - first_cov_page,
-	      fontmap->private->cov_pages,
-	      fontmap->private->num_cov_pages * sizeof (int *));
-      for (i = fontmap->private->first_cov_page + fontmap->private->num_cov_pages -
-	     first_cov_page; i < num_cov_pages; i++)
-	cov_pages[i] = NULL;
-
-      fontmap->private->first_cov_page = first_cov_page;
-      fontmap->private->num_cov_pages = num_cov_pages;
-      g_free (fontmap->private->cov_pages);
-      fontmap->private->cov_pages = cov_pages;
-
-      cov = g_new (int, 256);
-      for (i = 0; i < 256; i++)
-	cov[i] = -1;
-      fontmap->private->cov_pages[page - fontmap->private->first_cov_page] = cov;
-      cov_pages[page - first_cov_page] = cov;
-    }
-  else
-    {
-      cov = fontmap->private->cov_pages[page - fontmap->private->first_cov_page];
-      if (cov == NULL)
-	{
-	  cov = g_new (int, 256);
-	  for (i = 0; i < 256; i++)
-	    cov[i] = -1;
-	  fontmap->private->cov_pages[page - fontmap->private->first_cov_page] = cov;
-	}
-    }
-  cov[unicode & 0xff] = glyphnum;
-}
-#endif
-
 #define KERN_PAIR_HASH(g1, g2) ((g1) * 367 + (g2) * 31)
 
 static gboolean
@@ -1068,6 +979,18 @@ gff_load_afm (GnomeFontFace *face)
              for descenders below the baseline, but most people expect
              a positive number for descenders below the line. */
 	  face->private->descender = -fi->gfi->descender;
+	  face->private->fixed_width = fi->gfi->isFixedPitch;
+
+	  /* Added for pdf */
+	  face->private->capheight = (gdouble) fi->gfi->capHeight;
+	  face->private->italics_angle = (gdouble) fi->gfi->italicAngle;
+	  face->private->xheight = (gdouble) fi->gfi->xHeight;
+	  face->private->bbox.x0 = (gdouble) fi->gfi->fontBBox.llx;
+	  face->private->bbox.y0 = (gdouble) fi->gfi->fontBBox.lly;
+	  face->private->bbox.x1 = (gdouble) fi->gfi->fontBBox.urx;
+	  face->private->bbox.y1 = (gdouble) fi->gfi->fontBBox.ury;
+	  /* */
+	  
 	  face->private->underline_position = fi->gfi->underlinePosition;
 	  face->private->underline_thickness = fi->gfi->underlineThickness;
 
@@ -1116,13 +1039,12 @@ gff_load_afm (GnomeFontFace *face)
 		  }
 	  }
 
-	  g_renew (GFFGlyphInfo, face->private->glyphs, nglyphs);
+	  face->private->glyphs = g_renew (GFFGlyphInfo, face->private->glyphs, nglyphs);
 	  face->private->num_glyphs = nglyphs;
 
 	  for (i = 0; i < fi->numOfChars; i++)
 	    {
 	      int code;
-	      int unicode;
 
 	      /* Get the width */
 	      code = fi->cmi[i].code;
@@ -1148,18 +1070,6 @@ gff_load_afm (GnomeFontFace *face)
 			  ligtab[code] = ll;
 			}
 		    }
-
-#if 0
-		  /* add in the mapping from Unicode */
-		  unicode = gnome_font_find_unicode_encoding
-		    (gnome_font_type1_std_enc,
-		     sizeof(gnome_font_type1_std_enc) /
-		     sizeof(gnome_font_type1_std_enc[0]),
-		     fi->cmi[i].name);
-
-		  if (unicode > 0)
-		    gnome_font_add_unimapping (face, unicode, code);
-#endif
 		}
 	    }
 
@@ -1251,3 +1161,42 @@ gff_fill_zero_glyph (GnomeFontFace * face)
 }
 
 
+double
+gnome_font_face_get_capheight (const GnomeFontFace * face)
+{
+	g_return_val_if_fail (face != NULL, 0.0);
+	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), 0.0);
+	g_return_val_if_fail (GFF_METRICS (face), 0.0);
+
+	return face->private->capheight;
+}
+
+double
+gnome_font_face_get_italics_angle (const GnomeFontFace * face)
+{
+	g_return_val_if_fail (face != NULL, 0.0);
+	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), 0.0);
+	g_return_val_if_fail (GFF_METRICS (face), 0.0);
+
+	return face->private->italics_angle;
+}
+
+double
+gnome_font_face_get_xheight (const GnomeFontFace * face)
+{
+	g_return_val_if_fail (face != NULL, 0.0);
+	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), 0.0);
+	g_return_val_if_fail (GFF_METRICS (face), 0.0);
+
+	return face->private->xheight;
+}
+
+const ArtDRect *
+gnome_font_face_get_font_bbox (const GnomeFontFace * face)
+{
+	g_return_val_if_fail (face != NULL, NULL);
+	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
+	g_return_val_if_fail (GFF_METRICS (face), NULL);
+
+	return &face->private->bbox;
+}
