@@ -55,6 +55,7 @@ struct _GnomeFontFacePrivate {
 
 	GnomeFontWeight weight_code;
 	gboolean italic;
+	gboolean fixed_width;
 
 	/* Font metric info follows */
 	/* above is font metric info stored in the fontmap, below is font
@@ -68,6 +69,10 @@ struct _GnomeFontFacePrivate {
 	int   descender;
 	int   underline_position;
 	int   underline_thickness;
+	double capheight;
+	double italics_angle;
+	double xheight;
+	ArtDRect bbox;
 #if 0
 	int  *widths;
 #endif
@@ -113,6 +118,14 @@ gint gnome_font_face_get_num_glyphs (const GnomeFontFace * face);
 
 const gchar * gnome_font_face_get_glyph_ps_name (const GnomeFontFace * face, gint glyph);
 
+/*
+ * Return font caracteristics, grabbed from the AFM file, needed for PDF generation 
+ */
+double gnome_font_face_get_capheight    (const GnomeFontFace * face);
+double gnome_font_face_get_italics_angle (const GnomeFontFace * face);
+double gnome_font_face_get_xheight (const GnomeFontFace * face);
+const ArtDRect * gnome_font_face_get_font_bbox (const GnomeFontFace * face);
+
 END_GNOME_DECLS
 
 #endif
