@@ -31,6 +31,8 @@
 #include <libgnomeprint/gnome-print-encode.h>
 #endif
 
+#include "zlib.h"
+
 /* TODO : add a parameter to the encodeion methods called "size to beat" so that
    we stop encodeing a row that we know is too large to be chosen. We only need
    to return in_size+1 if the length of the current buffer beeing encodeed
@@ -43,6 +45,7 @@ int gnome_print_encode_hex     (const guchar *in, guchar *out, gint in_size);
 int gnome_print_decode_hex     (const guchar *in, guchar *out, gint in_size);
 int gnome_print_encode_ascii85 (const guchar *in, guchar *out, gint in_size);
 int gnome_print_decode_ascii85 (const guchar *in, guchar *out, gint in_size);
+int gnome_print_encode_deflate (const guchar *in, guchar *out, gint in_size, gint out_size);
 
 /* this worst case scenario functions return the size that the app should malloc
    for a buffer that will be encodeed with a particular algorithm */
@@ -53,6 +56,7 @@ int gnome_print_encode_hex_wcs     (gint size);
 int gnome_print_decode_hex_wcs     (gint size);
 int gnome_print_encode_ascii85_wcs (gint size);
 int gnome_print_decode_ascii85_wcs (gint size);
+int gnome_print_encode_deflate_wcs (gint size);
 
 #define GPC_MAX_CHARS_PER_LINE 80 /* Needs to be a multiple of 2 ! */
 
@@ -528,7 +532,7 @@ gint gnome_print_decode_hex_wcs (gint size)
 
 
 /* FIXME: Implement as a macro ? maybe */
-static gint
+static inline gint
 hex_2_dec (guchar upper, guchar lower)
 {
 	if (upper > '9')
@@ -829,6 +833,60 @@ int gnome_print_decode_ascii85 (const guchar *in, guchar *out, gint in_size)
 
 
 
+int
+gnome_print_encode_deflate_wcs (gint in_size)
+{
+	return (in_size * 1.01) + 24;
+}
+
+#define CHECK_ERR(err, msg) { \
+    if (err != Z_OK) { \
+        g_warning ("%s error: %d\n", msg, err); \
+        return -1; \
+    } \
+}
+
+int
+gnome_print_encode_deflate (const guchar *in, guchar *out, gint in_size, gint out_size)
+{
+	z_stream c_stream; /* compression stream */
+	int err;
+
+	c_stream.zalloc = (alloc_func)0;
+	c_stream.zfree = (free_func)0;
+	c_stream.opaque = (voidpf)0;
+	
+	err = deflateInit(&c_stream, Z_DEFAULT_COMPRESSION);
+	CHECK_ERR(err, "deflateInit");
+
+	c_stream.next_in  = (Bytef*)in;
+	c_stream.next_out = out;
+	c_stream.avail_in  = in_size;
+	c_stream.avail_out = out_size;
+
+	g_print ("In size %i out %i\n",
+		 in_size, out_size);
+
+	while (c_stream.total_in != (uLong)in_size &&
+	       c_stream.total_out < out_size) {
+		g_print ("loop 1\n");
+		err = deflate(&c_stream, Z_NO_FLUSH);
+		CHECK_ERR(err, "deflate");
+	}
+	/* Finish the stream, still forcing small buffers: */
+	for (;;) {
+		g_print ("Loop 2\n");
+		err = deflate(&c_stream, Z_FINISH);
+		if (err == Z_STREAM_END) break;
+		CHECK_ERR(err, "deflate");
+	}
+	
+	err = deflateEnd(&c_stream);
+	CHECK_ERR(err, "deflateEnd");
+
+	return c_stream.total_out;
+}
+	
 
 
 
@@ -881,37 +939,3 @@ gnome_print_encode_timer_end (void)
 }
 
 
-int
-gnome_print_encode_isolatin (const guint32 *in, guchar *out, gint length)
-{
-#if 0	
-	gint i;
-	guint32 u;
-	gchar *text;
-	text = g_new (gchar, 128);
-	text[0] = 0;
-	
-	for (i = 0; i < length; i++) {
-		u = GUINT32_FROM_BE (in[i]);
-		if (u == '(' || u == ')' || u == '\\') {
-			gnome_print_context_fprintf (pc, "\\%c", u);
-		} else if ( u == '-' ) {
-			/* This is a stupid Postscript bug in which all the chars below 127 are
-			   the same in the Standard Vector and the ISOLatin encodig BUT this one */
-			gnome_print_context_fprintf (pc, "\\255");
-		} else if ( u > 31 && u < 127) {
-			gnome_print_context_fprintf (pc, "%c", u);
-		} else if ( (u == 153) || (u == 156) || (u > 255) ) {
-			/* this chars are .notdef in isoLatin*/ 
-			gnome_print_context_fprintf (pc," ");
-		} else if ( (unsigned char) u > 143 ) {
-			gnome_print_context_fprintf (pc, "\\%o", u);
-		} else {
-			/* .notdef char */
-			gnome_print_context_fprintf (pc, " ");
-		}
-	}
-	
-#endif
-	return 0;
-}
