@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Metafile printer driver.
  *
@@ -26,10 +27,28 @@
 #include <string.h>
 #include <math.h>
 
+#include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-meta.h>
 
 #undef META_DEBUG
 
+typedef struct _GnomePrintMetaPrivate GnomePrintMetaPrivate;
+
+struct _GnomePrintMeta {
+	GnomePrintContext pc;
+
+	char *buffer;
+	int   buffer_size;
+	int   current;
+	int   pages;
+
+	GnomePrintMetaPrivate *priv;
+};
+
+struct _GnomePrintMetaClass {
+	GnomePrintContextClass parent_class;
+};
+
 static GnomePrintContextClass *parent_class = NULL;
 
 /* Note: this must continue to be the same length  as "GNOME_METAFILE-0.0" */
@@ -77,9 +96,11 @@ typedef enum {
 	GNOME_META_STROKEPATH,
 	GNOME_META_STROKE,
 	GNOME_META_SETFONT,
-	GNOME_META_SHOW,
+	GNOME_META_SHOW_UCS4,
 	GNOME_META_CONCAT,
+#ifdef SETMATRIX_DEPRECATED
 	GNOME_META_SETMATRIX,
+#endif	
 	GNOME_META_GSAVE,
 	GNOME_META_GRESTORE,
 	GNOME_META_CLIP,
@@ -112,9 +133,11 @@ static char *meta_type_names[] = {
 	"STROKEPATH",
 	"STROKE",
 	"SETFONT",
-	"SHOW",
+	"SHOW_UCS4",
 	"CONCAT",
+#ifdef SETMATRIX_DEPRECATED
 	"SETMATRIX",
+#endif	
 	"GSAVE",
 	"GRESTORE",
 	"CLIP",
@@ -461,28 +484,35 @@ meta_stroke (GnomePrintContext *pc)
 static int
 meta_setfont (GnomePrintContext *pc, GnomeFont *font)
 {
-	const char *fontname = font->fontmap_entry->font_name;
+	const char *fontname;
+
+	fontname = gnome_font_get_name (font);
 
 #ifdef META_DEBUG
 	g_warning ("SetFont starts at 0x%x",
 		   GNOME_PRINT_META (pc)->current);
 #endif
 	encode_int (pc, GNOME_META_SETFONT);
-	encode_double (pc, font->size);
+	encode_double (pc, gnome_font_get_size (font));
 	encode_string (pc, fontname);
 	return 0;
 }
 
 
 static int
-meta_show (GnomePrintContext *pc, char const *text)
+meta_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
 {
+	gint i;
+
 #ifdef META_DEBUG
-	g_warning ("Show starts at 0x%x",
+	g_warning ("Show_ucs4 starts at 0x%x",
 		   GNOME_PRINT_META (pc)->current);
 #endif
-	encode_int (pc, GNOME_META_SHOW);
-	encode_string (pc, text);
+	encode_int (pc, GNOME_META_SHOW_UCS4);
+	encode_int (pc, length);
+	for (i = 0; i < length; i++) {
+		encode_int (pc, GINT32_FROM_BE (buf[i]));
+	}
 	return 0;
 }
 
@@ -494,6 +524,7 @@ meta_concat (GnomePrintContext *pc, const double matrix [6])
 	return 0;
 }
 
+#ifdef SETMATRIX_DEPRECATED
 static int
 meta_setmatrix (GnomePrintContext *pc, const double matrix [6])
 {
@@ -501,6 +532,7 @@ meta_setmatrix (GnomePrintContext *pc, const double matrix [6])
 	encode_six_doubles (pc, matrix [0], matrix [1], matrix [2], matrix [3], matrix [4], matrix [5]);
 	return 0;
 }
+#endif
 
 static int
 meta_gsave (GnomePrintContext *pc)
@@ -666,9 +698,11 @@ gnome_print_meta_class_init (GnomePrintMetaClass *class)
 	pc_class->strokepath = meta_strokepath;
 	pc_class->stroke = meta_stroke;
 	pc_class->setfont = meta_setfont;
-	pc_class->show = meta_show;
+	pc_class->show_ucs4 = meta_show_ucs4;
 	pc_class->concat = meta_concat;
+#ifdef SETMATRIX_DEPRECATED
 	pc_class->setmatrix = meta_setmatrix;
+#endif	
 	pc_class->gsave = meta_gsave;
 	pc_class->grestore = meta_grestore;
 	pc_class->clip = meta_clip;
@@ -934,8 +968,10 @@ decode_image (const char *data, GnomePrintContext *dest, int bytes_per_pixel)
 
 	if (bytes_per_pixel == 1)
 		gnome_print_grayimage (dest, buffer, width, height, width);
-	else
+	else if (bytes_per_pixel == 3)
 		gnome_print_rgbimage (dest, buffer, width, height, width * bytes_per_pixel);
+	else
+		gnome_print_rgbaimage (dest, buffer, width, height, width * bytes_per_pixel);
 
 	g_free (buffer);
 	return data + size;
@@ -1125,27 +1161,42 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			break;
 		}
 
-		case GNOME_META_SHOW: {
-			char *s;
+		case GNOME_META_SHOW_UCS4: {
+			gint length;
+
+			data = decode_int (data, &length);
+
+			if (length > 0) {
+				guint32 * buf, val;
+				gint i;
+
+				buf = g_new (guint32, length);
+				for (i = 0; i < length; i++) {
+					data = decode_int (data, &val);
+					buf[i] = GUINT32_TO_BE (val);
+				}
+				gnome_print_show_ucs4 (dest, buf, length);
+				g_free (buf);
+			}
 
-			data = decode_string (data, &s);
-			gnome_print_show (dest, s);
-			g_free (s);
 			break;
 		}
 
 		case GNOME_META_CONCAT:
+#ifdef SETMATRIX_DEPRECATED
 		case GNOME_META_SETMATRIX:
+#endif	
 			data = decode_double (data, &matrix [0]);
 			data = decode_double (data, &matrix [1]);
 			data = decode_double (data, &matrix [2]);
 			data = decode_double (data, &matrix [3]);
 			data = decode_double (data, &matrix [4]);
 			data = decode_double (data, &matrix [5]);
-			if (opcode == GNOME_META_CONCAT)
-				gnome_print_concat (dest, matrix);
+			gnome_print_concat (dest, matrix);
+#ifdef SETMATRIX_DEPRECATED
 			else
 				gnome_print_setmatrix (dest, matrix);
+#endif	
 			break;
 
 		case GNOME_META_GSAVE:
@@ -1247,7 +1298,6 @@ gnome_print_meta_render (GnomePrintContext *destination, const void *meta_stream
 		g_warning ("This stream was not closed");
 		return FALSE;
 	}
-
 	return do_render (destination, data + PAGEHEADER_SIZE,
 			  header.size, -1);
 }
@@ -1265,12 +1315,29 @@ gnome_print_meta_render (GnomePrintContext *destination, const void *meta_stream
 gboolean
 gnome_print_meta_render_from_object (GnomePrintContext *destination, const GnomePrintMeta *source)
 {
+#if 1
+	const char * data;
+	GnomeMetaFileHeader header;
+#endif
 	g_return_val_if_fail (destination != NULL, FALSE);
 	g_return_val_if_fail (source != NULL, FALSE);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (destination), FALSE);
 	g_return_val_if_fail (GNOME_IS_PRINT_META (source), FALSE);
 
+#if 0
 	return gnome_print_meta_render (destination, source->buffer);
+#else
+	/* Render even unclosed metafiles */
+
+	data = source->buffer;
+	data = decode_header (data, &header);
+
+	if (strncmp (header.signature, GNOME_METAFILE_SIGNATURE, GNOME_METAFILE_SIGNATURE_SIZE) != 0)
+		return FALSE;
+
+	return do_render (destination, data + PAGEHEADER_SIZE,
+			  source->current - FILEHEADER_SIZE - PAGEHEADER_SIZE, -1);
+#endif
 }
 
 /**
