@@ -26,9 +26,27 @@
 #include <ctype.h>
 
 #include <libart_lgpl/art_affine.h>
+#include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-multipage.h>
 #include <libgnomeprint/gnome-font.h>
 
+typedef enum {
+  GNOME_PRINT_MULTIPAGE_TYPE_GENERIC_MULTIPAGE
+} GnomePrintMultipageType;
+
+struct _GnomePrintMultipage
+{
+  GnomePrintContext pc;
+
+  GnomePrintContext *subpc;
+  GList *affines; /* Of type double[6] */
+  GList *subpage;
+};
+
+struct _GnomePrintMultipageClass
+{
+  GnomePrintContextClass parent_class;
+};
 
 static void gnome_print_multipage_class_init (GnomePrintMultipageClass *klass);
 
@@ -189,10 +207,10 @@ gnome_print_multipage_setfont (GnomePrintContext *pc, GnomeFont *font)
 }
 
 static int
-gnome_print_multipage_show (GnomePrintContext *pc, char const *text)
+gnome_print_multipage_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
 {
   GnomePrintMultipage *multipage = GNOME_PRINT_MULTIPAGE(pc);
-  return gnome_print_show (multipage->subpc, text);
+  return gnome_print_show_ucs4 (multipage->subpc, buf, length);
 }
 
 static int
@@ -202,6 +220,7 @@ gnome_print_multipage_concat (GnomePrintContext *pc, const double matrix[6])
   return gnome_print_concat (multipage->subpc, matrix);
 }
 
+#ifdef SETMATRIX_DEPRECATED
 static int
 gnome_print_multipage_setmatrix (GnomePrintContext *pc, const double matrix[6])
 {
@@ -210,6 +229,7 @@ gnome_print_multipage_setmatrix (GnomePrintContext *pc, const double matrix[6])
   art_affine_multiply(affine, multipage->subpage->data, matrix);
   return gnome_print_setmatrix (multipage->subpc, affine);
 }
+#endif
 
 static int
 gnome_print_multipage_gsave (GnomePrintContext *pc)
@@ -329,9 +349,11 @@ gnome_print_multipage_class_init (GnomePrintMultipageClass *class)
   pc_class->strokepath = gnome_print_multipage_strokepath;
   pc_class->stroke = gnome_print_multipage_stroke;
   pc_class->setfont = gnome_print_multipage_setfont;
-  pc_class->show = gnome_print_multipage_show;
+  pc_class->show_ucs4 = gnome_print_multipage_show_ucs4;
   pc_class->concat = gnome_print_multipage_concat;
+#ifdef SETMATRIX_DEPRECATED
   pc_class->setmatrix = gnome_print_multipage_setmatrix;
+#endif	
   pc_class->gsave = gnome_print_multipage_gsave;
   pc_class->grestore = gnome_print_multipage_grestore;
   pc_class->clip = gnome_print_multipage_clip;
