@@ -22,12 +22,9 @@
 	minor / not for the first implementation :
 	- error while writing ( check for -1 )
 	- add page name to pdf file (set by begin page)
-	- conts variables
 	- add /Creator ( program name that created the PDF (ie. "gedit", "gnumeric")
-	- Compress the streams
 	- We need to check for maxline length while writing pages
 	  object numbers
-	- external text
 
 	later :
 	- add gnome_print_pdf_url & gnome_print_pdf_uri
@@ -36,7 +33,7 @@
 */
 
 
-/* __FUNCTION__ is not defined in Irix according to David Kaelbling <drk@sgi.com>*/
+/* __FUNCTION__ is not defined in Irix. thanks, <drk@sgi.com>*/
 #ifndef __GNUC__
   #define __FUNCTION__   ""
 #endif
@@ -57,10 +54,11 @@
 #include <libgnomeprint/gnome-printer-private.h>
 #include <libgnomeprint/gnome-font.h>
 #include <libgnomeprint/gnome-print-encode.h>
+#include <libgnomeprint/gnome-font-private.h>
 
 #define EOL "\r\n"
 #define GNOME_PRINT_PDF_BUFFER_SIZE 1024
-#define GNOME_PRINT_NUMBER_OF_ELEMENTS 16
+#define GNOME_PRINT_NUMBER_OF_ELEMENTS 32
 #define GNOME_PRINT_NUMBER_OF_ELEMENTS_GROW 8
 #define GNOME_PRINT_PDF_FONT_UNDEFINED 999
 
@@ -124,6 +122,8 @@ struct _GnomePrintPdfImage {
 struct _GnomePrintPdfFont {
 	GnomeFont *gnome_font;
 	guint      object_number;
+	guint      descriptor_object_number;
+	guint      pfb_object_number;
 	gint       font_number;
 	gchar     *font_name;
 	gboolean   is_basic_14;
@@ -478,22 +478,61 @@ gnome_print_pdf_page_start (GnomePrintContext *pc)
 static gint
 gnome_print_pdf_page_write_contents (GnomePrintContext *pc, GnomePrintPdfPage *page)
 {
+	GnomePrintPdf *pdf;
+	gboolean compress = FALSE;
+	gchar *compressed_stream = NULL;
 	gint ret = 0;
+	gint compressed_stream_length;
+	gint stream_length;
+	gint real_length;
+	
 
 	debug (FALSE, "");
 
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT(pc), -1);
 	g_return_val_if_fail (page != NULL, -1);
+	pdf = GNOME_PRINT_PDF (pc);
+	g_return_val_if_fail (GNOME_IS_PRINT_PDF (pdf), -1);
 
+	if (compress) {
+		/* Compress the buffer */
+		stream_length = strlen (page->stream);
+		compressed_stream_length = gnome_print_encode_deflate_wcs (stream_length);
+		compressed_stream	= g_malloc (compressed_stream_length);
+		real_length = gnome_print_encode_deflate (page->stream, compressed_stream,
+																							stream_length, compressed_stream_length);
+		/* */
+	}
+	
 	ret += gnome_print_pdf_object_start (pc, page->object_number_contents);
-	ret += gnome_print_pdf_write  (pc,"/Length %i" EOL, page->stream_used);
+	if (compress)
+		ret += gnome_print_pdf_write  (pc,"/Length %i" EOL, real_length + strlen(EOL));
+	else
+		ret += gnome_print_pdf_write  (pc,"/Length %i" EOL, page->stream_used);
+	if (compress) {
+  	ret += gnome_print_pdf_write  (pc,"/Filter ");
+		ret += gnome_print_pdf_write  (pc,"/FlateDecode ");
+		ret += gnome_print_pdf_write  (pc, EOL);
+	}
 	ret += gnome_print_pdf_write  (pc,">>" EOL);
 	ret += gnome_print_pdf_write  (pc,"stream" EOL);
-	ret += gnome_print_pdf_write  (pc,"%s", page->stream);
+
+	if (compress) {
+		pdf->offset += gnome_print_context_write_file (pc,
+																									 compressed_stream,
+																									 real_length);
+		ret += gnome_print_pdf_write  (pc, EOL);
+	}	else {
+		ret += gnome_print_pdf_write  (pc,"%s", page->stream);
+	}
+	
 	ret += gnome_print_pdf_write  (pc,"endstream" EOL);
 	ret += gnome_print_pdf_write  (pc,"endobj" EOL);
 	ret += gnome_print_pdf_object_end (pc, page->object_number_contents, TRUE);
 
+	if (compress)	
+		g_free (compressed_stream);
+
 	return ret;
 }
 
@@ -541,9 +580,12 @@ gnome_print_pdf_font_print_metrics (GnomePrintContext *pc, GnomePrintPdfFont *fo
 	gint first_char;
 	gint last_char;
 	guint32 n;
+	gint glyph;
 
-	return 0;
-	
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (font != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_FONT (font->gnome_font), -1);
+		
 	first_char = font->lowest;
 	last_char  = font->highest;
 
@@ -572,14 +614,14 @@ gnome_print_pdf_font_print_metrics (GnomePrintContext *pc, GnomePrintPdfFont *fo
 																 first_char,
 																 last_char);
 
-	for ( n = first_char; n < last_char ; n++)
+	for ( n = first_char; n <= last_char ; n++)
 	{
-		gnome_font_face_get_glyph_stdadvance (font_face, n, &point);
+		glyph = gnome_font_face_lookup_default (font_face, n);
+		
+		gnome_font_face_get_glyph_stdadvance (font_face, glyph, &point);
 
 		ret += gnome_print_pdf_write  (pc, "%g ",point.x);
 		
-		g_print ("x advance for char %i : is %g\n", n, point.x);
-		
 		if (((n+1)%16)==0)
 			ret += gnome_print_pdf_write  (pc, EOL);
 	}
@@ -589,6 +631,316 @@ gnome_print_pdf_font_print_metrics (GnomePrintContext *pc, GnomePrintPdfFont *fo
 	return ret;
 }
 
+
+static gint32
+gnome_font_face_get_pdf_flags (const GnomeFontFace *face)
+{
+	/* See the PDF 1.3 standard, section 7.11.2, page 225 */
+	gboolean fixed_width;
+	gboolean serif;
+	gboolean symbolic;
+	gboolean script;
+	gboolean italic;
+	gboolean all_cap;
+	gboolean small_cap;
+	gboolean force_bold;
+	gboolean adobe_roman_set;
+	gint32 flags;
+
+	fixed_width = gnome_font_face_is_fixed_width (face);
+	serif = TRUE;
+	symbolic = FALSE; /* FIXME */
+	script = FALSE;  /* FIXME */
+	italic = gnome_font_face_is_italic (face);
+	all_cap = FALSE;
+	small_cap = FALSE;
+	force_bold = FALSE;
+	adobe_roman_set = TRUE;
+
+	flags = 0;
+	flags =
+		fixed_width     |
+		(serif      << 1) |
+		(symbolic   << 2) |
+		(script     << 3) |
+		/* 5th bit is reserved */
+		(adobe_roman_set << 5) |
+		(italic     << 6) |
+		/* 8th - 16 are reserved */
+		(all_cap    << 16) |
+		(small_cap  << 17) |
+		(force_bold << 18);
+	/* 20-32 reserved */
+
+	return flags;
+}
+
+static gint
+gnome_font_face_get_stemv (const GnomeFontFace *face)
+{
+	/* FIXME */
+	return 96;
+}
+
+static gint
+text_utils_search_real (const gchar *buffer, gint buffer_length,
+												const gchar *search_text, gint search_text_length,
+												gboolean case_sensitive)
+{
+	gint p1;
+	gint p2 = 0;
+	gint case_sensitive_mask;
+
+	case_sensitive_mask = case_sensitive?0:32;
+	
+	for (p1=0; p1 < buffer_length; p1 ++)
+	{
+		if ((buffer[p1]     |case_sensitive_mask)==
+		    (search_text[p2]|case_sensitive_mask))
+		{
+			p2++;
+			if (p2==search_text_length)
+				return p1 - search_text_length + 1;
+		}
+		else
+			p2 = 0;
+	}
+	
+	return -1;
+}
+
+
+#if 0
+#define PDF_FONT_TAG_1 "%!"
+#define PDF_FONT_TAG_2 "currentfile eexec"
+#define PDF_FONT_TAG_3 "0000000000"
+
+static gint
+gnome_print_pdf_font_parse (const gchar *file_name,
+														gchar **body_,
+														gint *length, gint *length1, gint *length2, gint *length3)
+{
+	FILE *f;
+	gint ret = 0;
+	gchar *pfb;
+	gchar *pfb_encoded;
+	gint pfb_encoded_size;
+	gint pfb_size;
+	gint pfb_size_max;
+	gint bytes_read;
+	gint real_size;
+	gint pos1;
+	gint pos2;
+	gint pos3;
+
+  f = fopen (file_name, "r");
+  if (f == NULL)
+    {
+      g_warning ("Couldn't open font file %s\n", file_name);
+      return -1;
+    }
+  
+  pfb_size = 0;
+  pfb_size_max = 32768;
+  pfb = g_new (char, pfb_size_max);
+  while (1)
+    {
+      bytes_read = fread (pfb + pfb_size, 1, pfb_size_max - pfb_size, f);
+      if (bytes_read == 0) break;
+      pfb_size += bytes_read;
+      pfb = g_realloc (pfb, pfb_size_max <<= 1);
+    }
+
+	if (!pfb_size) {
+		g_warning ("Could not read the font in \"%s\"\n", file_name);
+		return -1;
+	}
+
+	g_print ("pfb_size %i\n", pfb_size);
+	
+	/* Search for %! */
+	pos1 = text_utils_search_real (pfb, pfb_size,
+																 PDF_FONT_TAG_1, strlen (PDF_FONT_TAG_1),
+																 TRUE);
+	
+	/* Search for "currentfile eexec"*/
+	pos2 = text_utils_search_real (pfb, pfb_size,
+																 PDF_FONT_TAG_2, strlen (PDF_FONT_TAG_2),
+																 TRUE);
+
+	/* Search for 00000 */
+	pos3 = text_utils_search_real (pfb, pfb_size,
+																 PDF_FONT_TAG_3, strlen (PDF_FONT_TAG_3),
+																 TRUE);
+	
+	g_print ("Pos1 :%i Pos2:%i Pos3:%i\n", pos1, pos2, pos3);
+	
+	pfb_encoded_size = gnome_print_encode_ascii85_wcs (pfb_size - pos1);
+	pfb_encoded = g_new (gchar, pfb_encoded_size);
+	real_size = gnome_print_encode_ascii85 (pfb + pos1,
+																					pfb_encoded,
+																					pos3 - pos2 - strlen (PDF_FONT_TAG_2) - 1
+																					/*pfb_size - pos1*/);
+	g_print ("Real Size %i\n", real_size);
+	
+	/* Get the lengths */
+	g_free (pfb);
+	
+	*body_ = pfb_encoded;
+	*length = real_size + strlen (EOL);
+	*length1 = pos2 + strlen (PDF_FONT_TAG_2) - pos1 + 1;
+	*length2 = pos3 - pos2 - strlen (PDF_FONT_TAG_2) - 1;
+	*length3 = pfb_size - pos3;
+	*length3 = 0;
+
+	*length1 -= 1;
+	*length2 += 1;
+	
+	g_print ("Length 1:%i 2:%i 3:%i\n",
+					 *length1, *length2, *length3);
+	
+	return ret;
+}
+#endif
+
+#if 0
+static gint
+gnome_print_pdf_font_embed (GnomePrintContext *pc,
+														GnomePrintPdfFont *font)
+{
+	const GnomeFontFace *face;
+	const gchar *file_name;
+	gchar *body;
+	gint length;
+	gint length1;
+	gint length2;
+	gint length3;
+	guint object_number;
+	gint ret = 0;
+	
+	debug (FALSE, "");
+
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (font != NULL, -1);
+	face = gnome_font_get_face (font->gnome_font);
+	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), -1);
+
+	file_name = face->private->pfb_fn;
+
+	gnome_print_pdf_font_parse (file_name,
+															&body,
+															&length,
+															&length1,
+															&length2,
+															&length3);
+															
+	object_number = font->pfb_object_number;
+
+	ret += gnome_print_pdf_object_start (pc, object_number);
+	ret += gnome_print_pdf_write  (pc,
+																 "/Filter /ASCII85Decode" EOL
+																 "/Length %i" EOL
+																 "/Length1 %i" EOL
+																 "/Length2 %i" EOL
+																 "/Length3 %i" EOL,
+																 length,
+																 length1,
+																 length2,
+																 length3);
+
+	ret += gnome_print_pdf_write  (pc,
+																 ">>" EOL
+																 "stream" EOL);
+
+	ret += gnome_print_pdf_write  (pc,
+																 "%s" EOL
+																 "endstream" EOL,
+																 body);
+
+	ret += gnome_print_pdf_object_end   (pc, object_number, TRUE);
+
+	g_free (body);
+
+	return ret;
+}
+#endif
+
+static gint
+gnome_print_pdf_font_print_descriptor (GnomePrintContext *pc,
+																			 GnomePrintPdfFont *font)
+{
+	const GnomeFontFace *face;
+	const ArtDRect *rect;
+	guint object_number;
+	guint pfb_object_number;
+	gint ret = 0;
+	gint ascent;
+	gint descent;
+	gint capheight;
+	gint flags;
+	gint italic_angle;
+	gint stemv;
+	gint xheight;
+	
+	debug (FALSE, "");
+
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (font != NULL, -1);
+	g_return_val_if_fail (font->gnome_font != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_FONT (font->gnome_font), -1);
+	face = gnome_font_get_face (font->gnome_font);
+	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), -1);
+
+	object_number     = gnome_print_pdf_object_number (pc);
+	pfb_object_number = gnome_print_pdf_object_number (pc);
+	font->descriptor_object_number = object_number;
+	font->pfb_object_number = pfb_object_number;
+	
+	ret += gnome_print_pdf_object_start (pc, object_number);
+
+	ascent = (gint) gnome_font_face_get_ascender (face);
+	descent = (gint) gnome_font_face_get_descender (face);
+	italic_angle = (gint) gnome_font_face_get_italics_angle (face);
+	rect = gnome_font_face_get_font_bbox (face);
+	capheight = (gint) gnome_font_face_get_capheight (face);
+	flags = gnome_font_face_get_pdf_flags (face);
+	stemv = gnome_font_face_get_stemv (face);
+	xheight = gnome_font_face_get_xheight (face);
+	
+	ret += gnome_print_pdf_write  (pc,
+																 "/Type /FontDescriptor" EOL
+																 "/Ascent %i" EOL
+																 "/CapHeight %i" EOL
+																 "/Descent -%i" EOL
+																 "/Flags %i" EOL
+																 "/FontBBox [%g %g %g %g]" EOL
+																 "/FontName /%s" EOL
+																 "/ItalicAngle %i" EOL
+																 "/StemV %i" EOL
+																 "/XHeight %i" EOL,
+#if 0																 
+																 "/FontFile %i 0 R" EOL,
+#endif	
+																 ascent,
+																 capheight,
+																 descent,
+																 flags,
+																 rect->x0, rect->y0, rect->x1, rect->y1,
+																 font->font_name,
+																 italic_angle,
+																 stemv,
+																 xheight
+#if 0																 
+																 pfb_object_number
+#endif
+		);
+
+	ret += gnome_print_pdf_object_end   (pc, object_number, FALSE);
+
+	return ret;
+}
+
+	
 static gint
 gnome_print_pdf_fonts (GnomePrintContext *pc)
 {
@@ -611,6 +963,14 @@ gnome_print_pdf_fonts (GnomePrintContext *pc)
 	for (n=0; n < pdf->fonts_number; n++)	{
 		font = &pdf->fonts [n];
 
+		if (!font->is_basic_14) {
+			/*
+			ret += gnome_print_pdf_font_print_descriptor (pc, font);
+			*/
+			/*
+			ret += gnome_print_pdf_font_embed            (pc, font);
+			*/
+		}
 		ret += gnome_print_pdf_object_start (pc, font->object_number);
 		ret += gnome_print_pdf_write  (pc,
 																	 "/Type /Font" EOL
@@ -625,6 +985,11 @@ gnome_print_pdf_fonts (GnomePrintContext *pc)
 																	 "/BaseFont /%s" EOL,
 																	 pdf->object_number_encoding,
 																	 font->font_name);
+		if (!font->is_basic_14)
+			ret += gnome_print_pdf_write  (pc,
+																		 "/FontDescriptor %i 0 R" EOL,
+																		 font->descriptor_object_number);
+		
 		ret += gnome_print_pdf_object_end   (pc, font->object_number, FALSE);
 	}
 
@@ -656,13 +1021,13 @@ gnome_print_pdf_error (gint fatal, const char *format, ...)
 static gint
 gnome_print_pdf_images (GnomePrintContext *pc, GnomePrintPdfPage *page)
 {
-	GnomePrintPdf *pdf;
 	GnomePrintPdfImage *image;
+	GnomePrintPdf *pdf;
+	const gchar type_gray[] = "DeviceGray";
+	const gchar type_rgb [] = "DeviceRGB";
+	const gchar *image_type_text;
 	gint ret = 0;
 	gint n;
-	gchar type_gray[] = "DeviceGray";
-	gchar type_rgb [] = "DeviceRGB";
-	gchar *image_type_text;
 	
 	debug (FALSE, "");
 
@@ -688,7 +1053,7 @@ gnome_print_pdf_images (GnomePrintContext *pc, GnomePrintPdfPage *page)
 			image_type_text = type_gray;
 			break;
 		}
-		
+
 		ret += gnome_print_pdf_object_start (pc, image->object_number);
 		ret += gnome_print_pdf_write  (pc,
 																	 "/Type /XObject" EOL
@@ -697,20 +1062,31 @@ gnome_print_pdf_images (GnomePrintContext *pc, GnomePrintPdfPage *page)
 																	 "/Width %i" EOL
 																	 "/Height %i" EOL
 																	 "/BitsPerComponent 8" EOL
-																	 "/ColorSpace /%s" EOL
-																	 "/Length %i" EOL
-																	 "/Filter /ASCII85Decode" EOL
-																	 ">>" EOL
-																	 "stream" EOL
-																	 "%s" EOL
-																	 "endstream" EOL
-																	 "endobj" EOL,
+																	 "/ColorSpace /%s" EOL,
 																	 image->image_number,
 																	 image->width,
 																	 image->height,
-																	 image_type_text,
-																	 image->data_length,
-																	 image->data);
+																	 image_type_text);
+		ret += gnome_print_pdf_write  (pc,
+																	 "/Length %i" EOL
+																	 "/Filter /FlateDecode" EOL,
+																	 image->data_length + strlen (EOL));
+		
+		ret += gnome_print_pdf_write  (pc,
+																	 ">>" EOL
+																	 "stream" EOL);
+
+		pdf->offset += gnome_print_context_write_file (pc,
+																									 image->data,
+																									 image->data_length);
+		
+		ret += gnome_print_pdf_write  (pc, EOL);
+		
+		ret += gnome_print_pdf_write  (pc,
+																	 "endstream" EOL
+																	 "endobj" EOL);
+
+		
 		ret += gnome_print_pdf_object_end   (pc, image->object_number, TRUE);
 	}
 
@@ -856,7 +1232,6 @@ gnome_print_pdf_page_end (GnomePrintContext *pc)
 	pdf->current_page = NULL;
 
 	pdf->graphic_state->written = FALSE;
-
 	
 	return ret;
 
@@ -875,7 +1250,7 @@ static gboolean
 gnome_print_dash_init (GnomePrintDash *dash)
 {
 	debug (FALSE, "");
-	
+
 	dash->number_values = 0;
 	dash->phase = 0;
 	dash->values = 0;
@@ -969,8 +1344,8 @@ gnome_print_pdf_showpage (GnomePrintContext *pc)
 	/* We need to clean the Graphic State */
 	gnome_print_pdf_graphic_state_free (pdf->graphic_state);
 	gnome_print_pdf_graphic_state_free (pdf->graphic_state_set);
-	pdf->graphic_state = gnome_print_pdf_graphic_state_new (FALSE);
-	pdf->graphic_state = gnome_print_pdf_graphic_state_new (TRUE);
+	pdf->graphic_state     = gnome_print_pdf_graphic_state_new (FALSE);
+	pdf->graphic_state_set = gnome_print_pdf_graphic_state_new (TRUE);
 	
 	gnome_print_pdf_page_start (pc);
 		
@@ -1041,6 +1416,7 @@ gnome_print_pdf_id_new (GnomePrintPdf *pdf)
 	
 	debug (FALSE, "");
 
+	/* FIXME: Improve ID generation */
 	a = g_strdup_printf ("%.12d%.2d%.12d%.6ld",
 											 (gint) time(NULL),
 											 95,
@@ -1157,6 +1533,7 @@ gnome_print_pdf_pages (GnomePrintContext *pc)
 	}
 	g_list_free (list);
 
+	/* FIXME: Paper Size is Hardcoded */
 	ret += gnome_print_pdf_write (pc,
 																"]" EOL
 																"/Count %i" EOL
@@ -1369,7 +1746,7 @@ gnome_print_pdf_write_content (GnomePrintPdf *pdf, const char *format, ...)
 		page->stream_allocated += GNOME_PRINT_PDF_BUFFER_SIZE;
 		page->stream = g_realloc (page->stream, page->stream_allocated);
 	}
-		
+
 	memcpy (page->stream + page->stream_used,
 					text,
 					text_length * sizeof (gchar));
@@ -1433,6 +1810,7 @@ gnome_print_pdf_setrgbcolor (GnomePrintContext *pc,
 	pdf = GNOME_PRINT_PDF (pc);
 	g_return_val_if_fail (pdf != NULL, -1);
 
+	
 	gs = gnome_print_pdf_graphic_state_current (pdf, TRUE);
 
 	gs->color_fill_mode   = PDF_COLOR_MODE_DEVICERGB;
@@ -1590,7 +1968,7 @@ gnome_print_dash_copy (GnomePrintDash *dash_from, GnomePrintDash *dash_to)
 	if (dash_to->values == NULL)
 		dash_to->values = g_new (gdouble, dash_from->number_values);
 	
-	memcpy (dash_to->values,
+if(FALSE)	memcpy (dash_to->values,
 					dash_from->values,
 					dash_to->number_values * sizeof (gdouble));
 }
@@ -1944,6 +2322,8 @@ gnome_print_pdf_graphic_state_duplicate (GnomePrintPdfGraphicState *gs_in)
 {
 	GnomePrintPdfGraphicState *gs_out;
 
+	g_return_val_if_fail (gs_in != NULL, NULL);
+	
 	debug (FALSE, "");
 	
 	gs_out = g_new (GnomePrintPdfGraphicState, 1);
@@ -2018,12 +2398,22 @@ gnome_print_pdf_grestore (GnomePrintContext *pc)
 	return ret;
 }
 
+
+static inline gboolean
+gp_zero_clean (double cleanme)
+{
+	if (abs(cleanme) > 0.00000001)
+		return cleanme;
+	return 0.0;
+}
+			
 static gint
 gnome_print_pdf_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
 {
 	GnomePrintPdf *pdf;
 	GnomePrintPdfGraphicState *gs;
 	ArtPoint point;
+	double val_1, val_2, val_3, val_4;
 	gint ret = 0;
 	gint i;
   guint32 u;
@@ -2045,6 +2435,11 @@ gnome_print_pdf_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
 		gnome_print_pdf_error (FALSE, "show, fontname or fontsize not defined.");
 		return -1;
 	}
+
+	val_1 = gp_zero_clean (gs->font_size * gs->ctm[0]);
+	val_2 = gp_zero_clean (gs->font_size * gs->ctm[1]);
+	val_3 = gp_zero_clean (gs->font_size * gs->ctm[2]);
+	val_4 = gp_zero_clean (gs->font_size * gs->ctm[3]);
 	
 	gp_path_currentpoint (gs->current_path, &point);
 
@@ -2052,10 +2447,10 @@ gnome_print_pdf_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
 	ret += gnome_print_pdf_graphic_state_set_font (pdf);
 	ret += gnome_print_pdf_write_content (pdf,
 																				"%g %g %g %g %g %g Tm" EOL,
-																				gs->font_size * gs->ctm[0],
-																				gs->font_size * gs->ctm[1],
-																				gs->font_size * gs->ctm[2],
-																				gs->font_size * gs->ctm[3],
+																				val_1,
+																				val_2,
+																				val_3,
+																				val_4,
 																				point.x,
 																				point.y);
 	gnome_print_pdf_graphic_state_text_set (pdf);
@@ -2069,15 +2464,9 @@ gnome_print_pdf_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
     u = GUINT32_FROM_BE (buf[i]);
 
 		if (u <	pdf->fonts[gs->pdf_font_number].lowest )
-		{
 			pdf->fonts[gs->pdf_font_number].lowest = u;
-			g_print ("Lowest set to :%i\n", u);
-		}
 		if (u > pdf->fonts[gs->pdf_font_number].highest)
-		{
-			g_print ("Highest set to :%i\n", u);
 			pdf->fonts[gs->pdf_font_number].highest = u;
-		}
 			
 		if (u == '(' || u == ')' || u == '\\') 
 		{
@@ -2350,6 +2739,9 @@ gnome_print_pdf_get_font_number (GnomePrintContext *pc, GnomeFont *gnome_font,
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	pdf = GNOME_PRINT_PDF (pc);
 	g_return_val_if_fail (pdf != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_FONT (gnome_font), -1);
+
+	g_print ("Yes\n");
 	
 	font_name = g_strdup (gnome_font_get_ps_name (gnome_font));
 
@@ -2371,6 +2763,8 @@ gnome_print_pdf_get_font_number (GnomePrintContext *pc, GnomeFont *gnome_font,
 	font->gnome_font    = gnome_font;
 	font->font_name     = font_name;
 	font->object_number = 0;
+	font->descriptor_object_number = 5890990;
+	font->pfb_object_number        = 2931915;
 	font->is_basic_14   = is_basic_14;
 	font->lowest        = 255; 
 	font->highest       = 0;
@@ -2405,12 +2799,6 @@ gnome_print_pdf_setfont (GnomePrintContext *pc, GnomeFont *font)
       break;
 
 	if (n == pdf->fonts_internal_number) {
-		static gint warned = FALSE;
-		if (!warned) {
-			g_warning ("External fonts not implemented.");
-			warned = TRUE;
-		}
-		return 0;
 		is_basic_14 = FALSE;
 	}	else {
 		is_basic_14 = pdf->fonts_internal[n].is_basic_14;
@@ -2561,7 +2949,8 @@ gnome_print_pdf_closepath (GnomePrintContext *pc)
 
 static gint
 gnome_print_pdf_image_load (GnomePrintPdf *pdf, gchar *data, gint width, gint height,
-														gint rowstride, gint bytes_per_pixel, gint image_type)
+														gint rowstride, gint bytes_per_pixel, gint image_type,
+														gint data_length)
 {
 	GnomePrintPdfImage *image;
 	GnomePrintPdfPage *page;
@@ -2576,9 +2965,9 @@ gnome_print_pdf_image_load (GnomePrintPdf *pdf, gchar *data, gint width, gint he
 															(page->images_max += GNOME_PRINT_NUMBER_OF_ELEMENTS_GROW));
 
 	image = &page->images[page->images_number++];
-	
-	image->data = g_strdup (data);
-	image->data_length = strlen (data); /* the stream data, migth be compresed */
+	image->data = g_malloc (data_length+1);
+	memcpy (image->data, data, data_length);
+	image->data_length = data_length;
 	image->width = width;
 	image->height = height;
 	image->rowstride = rowstride;
@@ -2591,16 +2980,18 @@ gnome_print_pdf_image_load (GnomePrintPdf *pdf, gchar *data, gint width, gint he
 }
 
 static gint
-gnome_print_pdf_image_compressed (GnomePrintContext *pc, const char *data, gint width, gint height,
-																	gint rowstride, gint bytes_per_pixel, gint image_type)
+gnome_print_pdf_image_compressed (GnomePrintContext *pc,
+																	const char *data,
+																	gint width,
+																	gint height,
+																	gint rowstride,
+																	gint bytes_per_pixel,
+																	gint image_type)
 {
 	GnomePrintPdf *pdf;
 	GnomePrintPdfGraphicState *gs;
 	gint ret = 0;
 	gint image_number;
-	gint n;
-	
-
 	gchar *image_stream;
 	gint   image_stream_size;
 
@@ -2616,19 +3007,25 @@ gnome_print_pdf_image_compressed (GnomePrintContext *pc, const char *data, gint
 																				gs->ctm[0], gs->ctm[1],
 																				gs->ctm[2],	gs->ctm[3],
 																				gs->ctm[4], gs->ctm[5]);
-
+#if 0	
 	image_stream_size = gnome_print_encode_ascii85_wcs (width * height * bytes_per_pixel);
+#else	
+	image_stream_size = gnome_print_encode_deflate_wcs (width * height * bytes_per_pixel);
+#endif	
 	image_stream      = g_new (gchar, image_stream_size);
 
-	gnome_print_encode_timer_start ();
-	for (n=0;n<100;n++)
+#if 0	
 		image_stream_size = gnome_print_encode_ascii85 (data, image_stream, width * height * bytes_per_pixel);
-	gnome_print_encode_timer_end ();
+#else	
+		image_stream_size = gnome_print_encode_deflate (data, image_stream,
+																									width * height * bytes_per_pixel,
+																									image_stream_size);
+#endif	
 
 	image_number      = gnome_print_pdf_image_load (pdf, image_stream,
 																									width, height,
 																									rowstride, bytes_per_pixel,
-																									image_type);
+																									image_type, image_stream_size);
 
 	ret += gnome_print_pdf_write_content (pdf, "/Im%i Do" EOL,
 																				image_number);
@@ -2810,7 +3207,7 @@ gnome_print_pdf_setfont_raw (GnomePrintContext *pc, GnomeFontFace * face,
 	    pdf->fonts = g_realloc (pdf->fonts, sizeof (char *) *
 															(pdf->n_fonts_max <<= 1));
 	  pdf->fonts[pdf->n_fonts++] = g_strdup (fontname);
-	  g_free (pfa);
+		g_free (pfa);
 	  gnome_print_pdf_reencode_font (pc, fontname);
 	}
     }
