@@ -2,59 +2,34 @@
  * gnome-print-preview.c: A preview driver for GnomePrint that renders
  * into a GnomeCanvas and a control object for the pages rendered.
  *
- * Author:
+ * Authors:
  *   Miguel de Icaza (miguel@gnu.org)
+ *   Lauris Kaplinski (lauris@ariman.ee)
  *
  * (C) 1999, 2000 International GNOME Support
  */
+
 #include <config.h>
 #include <gtk/gtk.h>
 #include <string.h>
 #include <math.h>
 
 #include <libgnomeui/gnome-canvas-image.h>
+#include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-preview.h>
+#include <libgnomeprint/gnome-print-preview-private.h>
 #include <libgnomeprint/gnome-canvas-bpath.h>
 #include <libgnomeprint/gnome-canvas-hacktext.h>
+#include <libgnomeprint/gnome-canvas-clipgroup.h>
 #include <libgnomeprint/gnome-font.h>
 #include <libgnomeprint/gt1-parset1.h>
 #include <libart_lgpl/art_pixbuf.h>
 #include <libgnome/gnome-paper.h>
 
-typedef struct {
-	double x, y;
-} coord_t;
-
-typedef struct {
-	double r, g, b;
-	int    opacity;
-} color_t;
-
-typedef struct {
-	color_t      color;
-	double       width;
-	int          opacity;
-	GdkCapStyle  linecap;
-	GdkJoinStyle join;
-
-	int          dash_n_values;
-	double      *dash_values;
-	double       dash_offset;
-	GnomeCanvasBpathDef *bpath;
-	GnomeCanvasGroup    *group;
-
-	GnomeFont    *font;
-
-	double       affine [6];
-	double       page_affine [6];
-	double       base_affine [6];
-	double       startup_page_affine [6];
-
-} GraphicContext;
+#include <libgnomeprint/gp-gc.h>
 
 struct _GnomePrintPreviewPrivate {
-	/* List of saved printing contexts */
-	GSList *context_list;
+	GPGC * gc;
 
 	/* Current page displayed */
 	int top_page;
@@ -69,136 +44,24 @@ struct _GnomePrintPreviewPrivate {
 	GHashTable *font_hash;
 };
 
-#define gc(o) ((GraphicContext *)((o)->context_list->data))
-
 static GnomePrintContextClass *print_preview_parent_class;
 
-static void
-gc_set_dash (GraphicContext *ctx, int n_values, const double *values, double offset)
-{
-	ctx->dash_n_values = n_values;
-	ctx->dash_values   = g_new (double, n_values);
-	ctx->dash_offset   = offset;
-
-	memcpy (ctx->dash_values, values, sizeof (double) * n_values);
-}
-
-static void
-gc_clear_bpath (GraphicContext *ctx)
-{
-	if (!ctx->bpath)
-		return;
-	
-	gnome_canvas_bpath_def_unref (ctx->bpath);
-	ctx->bpath = 0;
-}
-
-static void
-gc_free (GraphicContext *ctx)
-{
-	if (ctx->dash_values)
-		g_free (ctx->dash_values);
-	if (ctx->font)
-		gtk_object_unref (GTK_OBJECT (ctx->font));
-	if (ctx->bpath)
-		gc_clear_bpath (ctx);
-	g_free (ctx);
-}
-
-static void
-set_color (color_t *color, double r, double g, double b, int opacity)
-{
-	color->r = r;
-	color->g = g;
-	color->b = b;
-
-	if (opacity != -1)
-		color->opacity = opacity;
-}
-
-static gint32
-color_rgba (color_t *color)
-{
-	return ((((guint)(color->r * 255)) << 24) |
-		(((guint)(color->g * 255)) << 16) |
-		(((guint)(color->b * 255)) << 8)) |
-		color->opacity;
-	
-}
-
-#if 0
-static void
-dump_gc (GnomePrintPreview *pp)
-{
-	GnomePrintPreviewPrivate *priv = pp->priv;
-	GraphicContext *g = gc (priv);
-	
-	printf ("color:      %08x\n", color_rgba (&g->color));
-	printf ("width:      %g\n",   g->width);
-	printf ("opacity:    %d\n",   g->opacity);
-	printf ("linecap:    %d\n",   g->linecap);
-	printf ("join:       %d\n",   g->join);
-	printf ("affine:     %g %g %g %g %g %g\n",
-		g->affine [0],
-		g->affine [1],
-		g->affine [2],
-		g->affine [3],
-		g->affine [4],
-		g->affine [5]);
-
-	if (g->bpath) {
-		int i;
-		
-		for (i = 0; i < g->bpath->n_bpath; i++) {
-
-		}
-	}
-}
-#endif
-
 static int
 gpp_newpath (GnomePrintContext *pc)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
-	
-	if (gc (priv)->bpath)
-		gc_clear_bpath (gc (priv));
 
-	gc (priv)->bpath = gnome_canvas_bpath_def_new ();
+	gp_gc_newpath (pp->priv->gc);
 
 	return 1;
 }
 
-static void
-map (GnomePrintPreviewPrivate *priv, double *x, double *y)
-{
-	ArtPoint p;
-
-	p.x = *x;
-	p.y = *y;
-
-	/* Apply postscript affine transform */
-	art_affine_point (&p, &p, gc (priv)->affine);
-
-	/* Map PS to canvas coordinates */
-	art_affine_point (&p, &p, gc (priv)->page_affine);
-
-	*x = p.x;
-	*y = p.y;
-}
-
 static int
 gpp_moveto (GnomePrintContext *pc, double x, double y)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 	
-	if (gc (priv)->bpath == NULL)
-		gc (priv)->bpath = gnome_canvas_bpath_def_new ();
-
-	map (priv, &x, &y);
-	gnome_canvas_bpath_def_moveto (gc (priv)->bpath, x, y);
+	gp_gc_moveto (pp->priv->gc, x, y);
 
 	return 1;
 }
@@ -207,12 +70,10 @@ static int
 gpp_lineto (GnomePrintContext *pc, double x, double y)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	g_return_val_if_fail (gc (priv)->bpath != NULL, -1);
+	g_return_val_if_fail (gp_gc_has_currentpoint (pp->priv->gc), -1);
 	
-	map (priv, &x, &y);
-	gnome_canvas_bpath_def_lineto (gc (priv)->bpath, x, y);
+	gp_gc_lineto (pp->priv->gc, x, y);
 
 	return 1;
 }
@@ -224,14 +85,10 @@ gpp_curveto (GnomePrintContext *pc,
 	     double x3, double y3)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	g_return_val_if_fail (gc (priv)->bpath != NULL, -1);
+	g_return_val_if_fail (gp_gc_has_currentpoint (pp->priv->gc), -1);
 	
-	map (priv, &x1, &y1);
-	map (priv, &x2, &y2);
-	map (priv, &x3, &y3);
-	gnome_canvas_bpath_def_curveto (gc (priv)->bpath, x1, y1, x2, y2, x3, y3);
+	gp_gc_curveto (pp->priv->gc, x1, y1, x2, y2, x3, y3);
 
 	return 1;
 }
@@ -240,11 +97,13 @@ static int
 gpp_closepath (GnomePrintContext *pc)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	g_return_val_if_fail (gc (priv)->bpath != NULL, -1);
+#if 0
+	/* fixme: ? */
+	g_return_val_if_fail (gp_path_length (priv->gc->path) > 2, -1);
+#endif
 
-	gnome_canvas_bpath_def_closepath (gc (priv)->bpath);
+	gp_gc_closepath (pp->priv->gc);
 
 	return 1;
 }
@@ -253,57 +112,43 @@ static int
 gpp_setrgbcolor (GnomePrintContext *pc, double r, double g, double b)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	set_color (&gc (priv)->color, r, g, b, - 1);
+	gp_gc_set_rgbcolor (pp->priv->gc, r, g, b);
 
 	return 1;
 }
 
-static void
-close_path (GnomeCanvasBpathDef *bpath)
-{
-	if (bpath->moveto_idx == -1)
-		return;
-	gnome_canvas_bpath_def_closepath (bpath);
-}
-
 static int
 gpp_stroke (GnomePrintContext *pc)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
 	GnomePrintPreviewPrivate *priv = pp->priv;
+	GnomeCanvasGroup *group;
 	GnomeCanvasItem *item;
-	GraphicContext *g = gc (priv);
-	int close;
-	
-	if (g->bpath == NULL)
-		return 0;
 
-	/*
-	 * Special case: libart does not like stroking of lines
-	 */
-	close = 1;
-	if (g->bpath->n_bpath == 2) {
-		if ((g->bpath->bpath [0].code == ART_MOVETO ||
-		     g->bpath->bpath [0].code == ART_MOVETO_OPEN) &&
-		    g->bpath->bpath [1].code == ART_LINETO)
-			close = 0;
-	}
+	/* fixme: currentpath invariants */
 
-	if (close)
-		close_path (g->bpath);
+	group = (GnomeCanvasGroup *) gp_gc_get_data (priv->gc);
+	g_assert (group != NULL);
+	g_assert (GNOME_IS_CANVAS_GROUP (group));
 
-	item = gnome_canvas_item_new (g->group,
+	/* Fixme: Can we assume that linewidth == width_units? */
+	/* Probably yes, as object->page ctm == identity */
+
+	item = gnome_canvas_item_new (group,
 		gnome_canvas_bpath_get_type (),
-		"bpath",       g->bpath,
-		"width_units", g->width,
-		"cap_style",   g->linecap,
-		"join_style",  g->join,
-		"outline_color_rgba", color_rgba (&g->color),
+		"bpath",	gp_gc_get_currentpath (priv->gc),
+		"width_units",	gp_gc_get_linewidth (priv->gc),
+		"cap_style",	gp_gc_get_linecap (priv->gc),
+		"join_style",	gp_gc_get_linejoin (priv->gc),
+		"outline_color_rgba", gp_gc_get_rgba (priv->gc),
+		"miterlimit",	gp_gc_get_miterlimit (priv->gc),
+		"dash",		gp_gc_get_dash (priv->gc),
 		NULL);
 
-	gc_clear_bpath (g);
+	/* fixme: is this correct? */
+	gp_gc_newpath (priv->gc);
+
 	return 1;
 }
 
@@ -311,24 +156,44 @@ static int
 gpp_strokepath (GnomePrintContext *pc)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
+
+	gp_gc_strokepath (pp->priv->gc);
+
+	return 1;
+}
+
+static int
+gpp_create_fill (GnomePrintPreviewPrivate * priv, ArtWindRule wind)
+{
+	GPPath * closedpath;
+	GnomeCanvasGroup *group;
 	GnomeCanvasItem *item;
 	
-	g_return_val_if_fail (gc (priv)->bpath != NULL, -1);
+	/* fixme: currentpath invariants */
 
-	close_path (gc (priv)->bpath);
-	g_warning ("Strokepath is not tested");
+	closedpath = gp_path_close_all (gp_gc_get_currentpath (priv->gc));
+	if (gp_path_is_empty (closedpath)) {
+		g_warning ("gpp_create_fill:closepath is empty");
+		gp_path_unref (closedpath);
+		return -1;
+	}
 
-	item = gnome_canvas_item_new (
-		gc (priv)->group,
+	group = (GnomeCanvasGroup *) gp_gc_get_data (priv->gc);
+	g_assert (group != NULL);
+	g_assert (GNOME_IS_CANVAS_GROUP (group));
+
+	item = gnome_canvas_item_new (group,
 		gnome_canvas_bpath_get_type (),
-		"bpath",       gc (priv)->bpath,
-		"width_units", gc (priv)->width,
-		"cap_style",   gc (priv)->linecap,
-		"join_style",  gc (priv)->join,
-		"outline_color_rgba", color_rgba (&gc (priv)->color),
+		"bpath", closedpath,
+		"outline_color", NULL,
+		"fill_color_rgba", gp_gc_get_rgba (priv->gc),
+		"wind", wind,
 		NULL);
-	gc_clear_bpath (gc (priv));
+
+	gp_path_unref (closedpath);
+
+	gp_gc_newpath (priv->gc);
+
 	return 1;
 }
 
@@ -336,45 +201,24 @@ static int
 gpp_fill (GnomePrintContext *pc)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
-	GnomeCanvasItem *item;
-	
-	g_return_val_if_fail (gc (priv)->bpath != NULL, -1);
 
-	close_path (gc (priv)->bpath);
-
-	item = gnome_canvas_item_new (
-		gc (priv)->group,
-		gnome_canvas_bpath_get_type (),
-		"bpath",       gc (priv)->bpath,
-		"width_units", gc (priv)->width,
-		"cap_style",   gc (priv)->linecap,
-		"join_style",  gc (priv)->join,
-		"fill_color_rgba", color_rgba (&gc (priv)->color),
-		NULL);
-	gc_clear_bpath (gc (priv));
-	return 1;
+	return gpp_create_fill (pp->priv, ART_WIND_RULE_NONZERO);
 }
 
 static int
 gpp_eofill (GnomePrintContext *pc)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	g_warning ("eofill operator not implemented yet");
-	gc_clear_bpath (gc (priv));
-
-	return 1;
+	return gpp_create_fill (pp->priv, ART_WIND_RULE_ODDEVEN);
 }
 
 static int
 gpp_setlinewidth (GnomePrintContext *pc, double width)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	gc (priv)->width = width;
+	gp_gc_set_linewidth (pp->priv->gc, width);
 
 	return 1;
 }
@@ -382,21 +226,33 @@ gpp_setlinewidth (GnomePrintContext *pc, double width)
 static int
 gpp_setmiterlimit (GnomePrintContext *pc, double limit)
 {
-	g_warning ("WOOOOO  What does SetMiterLimit do?");
-	return -1;
+	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
+
+	gp_gc_set_miterlimit (pp->priv->gc, limit);
+
+	return 1;
 }
 
 static int
 gpp_setlinejoin (GnomePrintContext *pc, int jointype)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
 	switch (jointype) {
+	case 0:
+		gp_gc_set_linejoin (pp->priv->gc, ART_PATH_STROKE_JOIN_MITER);
+		break;
+	case 1:
+		gp_gc_set_linejoin (pp->priv->gc, ART_PATH_STROKE_JOIN_ROUND);
+		break;
+	case 2:
+		gp_gc_set_linejoin (pp->priv->gc, ART_PATH_STROKE_JOIN_BEVEL);
+		break;
 	default:
-		gc (priv)->join = GDK_JOIN_MITER;
-		g_warning ("Need to support all linejoin methods");
+		return -1;
+		break;
 	}
+
 	return 1;
 }
 
@@ -404,13 +260,22 @@ static int
 gpp_setlinecap (GnomePrintContext *pc, int captype)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
 	switch (captype) {
+	case 0:
+		gp_gc_set_linecap (pp->priv->gc, ART_PATH_STROKE_CAP_BUTT);
+		break;
+	case 1:
+		gp_gc_set_linecap (pp->priv->gc, ART_PATH_STROKE_CAP_ROUND);
+		break;
+	case 2:
+		gp_gc_set_linecap (pp->priv->gc, ART_PATH_STROKE_CAP_SQUARE);
+		break;
 	default:
-		gc (priv)->linecap = GDK_CAP_BUTT;
-		g_warning ("Need to support all linecap methods");
+		return -1;
+		break;
 	}
+
 	return 1;
 }
 
@@ -418,12 +283,8 @@ static int
 gpp_setdash (GnomePrintContext *pc, int n_values, const double *values, double offset)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	if (gc (priv)->dash_values)
-		g_free (gc (priv)->dash_values);
-
-	gc_set_dash (gc (priv), n_values, values, offset);
+	gp_gc_set_dash (pp->priv->gc, n_values, values, offset);
 
 	return 1;
 }
@@ -432,13 +293,8 @@ static int
 gpp_setfont (GnomePrintContext *pc, GnomeFont *font)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	if (gc (priv)->font)
-		gtk_object_unref (GTK_OBJECT (gc (priv)->font));
-
-	gc (priv)->font = font;
-	gtk_object_ref (GTK_OBJECT (font));
+	gp_gc_set_font (pp->priv->gc, font);
 
 	return 1;
 }
@@ -447,103 +303,75 @@ static int
 gpp_concat (GnomePrintContext *pc, const double matrix [6])
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	art_affine_multiply (gc (priv)->affine,  matrix, gc (priv)->affine);
+	gp_gc_concat (pp->priv->gc, matrix);
 
 	return 1;
 }
 
+/* fixme: should we prepeng root page matrix? */
+#ifdef SETMATRIX_DEPRECATED
 static int
 gpp_setmatrix (GnomePrintContext *pc, const double matrix[6])
 {
+	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
+
 	g_warning ("setmatrix is deprecated");
 
-	return -1;
-}
+	gp_gc_setmatrix (pp->priv->gc, matrix);
 
-static GnomeCanvasBpathDef *
-gnome_canvas_bpath_def_duplicate (GnomeCanvasBpathDef *def)
-{
-	GnomeCanvasBpathDef *bpd;
-
-	g_return_val_if_fail (def != NULL, NULL);
-
-	if (def->n_bpath <= 0)
-		return gnome_canvas_bpath_def_new ();
-
-	bpd = g_new (GnomeCanvasBpathDef, 1);
-	
-	bpd->n_bpath = def->n_bpath;
-	bpd->n_bpath_max = def->n_bpath;
-	bpd->moveto_idx = def->moveto_idx;
-	bpd->ref_count = 1;
-	bpd->bpath = g_new (ArtBpath, def->n_bpath);
-
-	memcpy (bpd->bpath, def->bpath, def->n_bpath * sizeof (ArtBpath));
-	return bpd;
+	return 1;
 }
+#endif
 
 static int
 gpp_gsave (GnomePrintContext *pc)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
-	GraphicContext *newctx;
-	GraphicContext *ctx = gc (priv);
 
-	newctx = g_new (GraphicContext, 1);
-	memcpy (newctx, ctx, sizeof (GraphicContext));
-
-	if (newctx->dash_values){
-		
-		gc_set_dash (newctx, ctx->dash_n_values, ctx->dash_values, ctx->dash_offset);
-	}
-
-	if (newctx->bpath){
-		GnomeCanvasBpathDef *newpath;
-
-		newpath = gnome_canvas_bpath_def_duplicate (ctx->bpath);
-
-		newctx->bpath = newpath;
-	}
-
-	if (newctx->font)
-		gtk_object_ref (GTK_OBJECT (newctx->font));
-
-	newctx->group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (
-		ctx->group,
-		gnome_canvas_group_get_type (),
-		NULL));
-	
-	priv->context_list = g_slist_prepend (priv->context_list, newctx);
-/*	g_warning ("ggp_gsave\n");*/
+	gp_gc_gsave (pp->priv->gc);
 
 	return 1;
 }
 
-static void
-pop_state (GnomePrintContext *pc)
-{
-	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
-
-	gc_free (gc (priv));
-	priv->context_list = g_slist_remove (priv->context_list, gc (priv));
-}
-
 static int
 gpp_grestore (GnomePrintContext *pc)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	if (priv->context_list->next == NULL){
-		g_warning ("grestore invoked on emtpy stack");
+	gp_gc_grestore (pp->priv->gc);
+
+	return 1;
+}
+
+static int
+gpp_create_clip (GnomePrintPreviewPrivate * priv, ArtWindRule wind)
+{
+	GPPath * closedpath;
+	GnomeCanvasGroup * group;
+	GnomeCanvasItem * clip;
+
+	closedpath = gp_path_close_all (gp_gc_get_currentpath (priv->gc));
+	if (gp_path_is_empty (closedpath)) {
+		g_warning ("gpp_create_clip: closedpath is empty");
+		gp_path_unref (closedpath);
 		return -1;
 	}
 
-	pop_state (pc);
+	group = (GnomeCanvasGroup *) gp_gc_get_data (priv->gc);
+
+	clip = gnome_canvas_item_new (group,
+		gnome_canvas_clipgroup_get_type (),
+		"path", closedpath,
+		"wind", wind,
+		NULL);
+
+	gp_path_unref (closedpath);
+
+	gp_gc_set_data (priv->gc, clip);
+
+	gp_gc_clip (priv->gc);
+
 	return 1;
 }
 
@@ -551,27 +379,23 @@ static int
 gpp_clip (GnomePrintContext *pc)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
-	GraphicContext *g = gc (priv);
 
-	gc_clear_bpath (g);
-/*	g_warning ("Implement clip");*/
-	return 1;
+	return gpp_create_clip (pp->priv, ART_WIND_RULE_NONZERO);
 }
 
 static int
 gpp_eoclip (GnomePrintContext *pc)
 {
-	g_warning ("Implement eoclip");
-	return 1;
+	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
+
+	return gpp_create_clip (pp->priv, ART_WIND_RULE_ODDEVEN);
 }
 
 static int
 gpp_image (GnomePrintContext *pc, const char *data, int width, int height, int rowstride, int bytes_per_pixel)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
-	GraphicContext *g = gc (priv);
+	GnomeCanvasGroup * group;
 	GnomeCanvasItem *item;
 	ArtPixBuf *pixbuf;
 	int size, bpp;
@@ -580,13 +404,14 @@ gpp_image (GnomePrintContext *pc, const char *data, int width, int height, int r
 	/*
 	 * We do convert gray scale images to RGB
 	 */
+
 	if (bytes_per_pixel == 1)
 		bpp = 3;
 	else
 		bpp = bytes_per_pixel;
 	
 	size = width * height * bpp;
-	dup = g_malloc (size);
+	dup = art_alloc (size);
 	if (!dup)
 		return -1;
 
@@ -616,8 +441,9 @@ gpp_image (GnomePrintContext *pc, const char *data, int width, int height, int r
 	} else
 		return -1;
 
-	item = gnome_canvas_item_new (
-		g->group,
+	group = (GnomeCanvasGroup *) gp_gc_get_data (pp->priv->gc);
+
+	item = gnome_canvas_item_new (group,
 		gnome_canvas_image_get_type (),
 		"pixbuf", pixbuf,
 		"x",      0.0,
@@ -631,14 +457,19 @@ gpp_image (GnomePrintContext *pc, const char *data, int width, int height, int r
 	/* Apply the transformation for the image */
 	{
 		double transform [6];
-		double ident [6];
 		double flip [6];
+#if 0
+		double ident [6];
 		
 		art_affine_identity (ident);
 		art_affine_flip (flip, ident, FALSE, TRUE);
 		art_affine_multiply (transform, flip, g->affine);
 		art_affine_multiply (transform, transform, g->page_affine);
-		
+#else
+		art_affine_scale (flip, 1.0, -1.0);
+#endif
+		art_affine_multiply (transform, flip, gp_gc_get_ctm (pp->priv->gc));
+
 		gnome_canvas_item_affine_relative (item, transform);
 	}
 	
@@ -658,6 +489,13 @@ gpp_rgbimage (GnomePrintContext *pc, const char *data, int width, int height, in
 }
 
 
+static int
+gpp_rgbaimage (GnomePrintContext *pc, const char *data, int width, int height, int rowstride)
+{
+	return gpp_image (pc, data, width, height, rowstride, 4);
+}
+
+
 static int
 gpp_textline (GnomePrintContext *pc, GnomeTextLine *line)
 {
@@ -670,8 +508,7 @@ static int
 gpp_showpage (GnomePrintContext *pc)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
-
+#if 0
 	/*
 	 * Reset affine and path
 	 */
@@ -681,16 +518,22 @@ gpp_showpage (GnomePrintContext *pc)
 	 * Kill current path
 	 */
 	gc_clear_bpath (gc (priv));
+#endif
+#ifdef SETMATRIX_DEPRECATED
+	gp_gc_setmatrix (pp->priv->gc, NULL);
+#endif	
+	gp_gc_newpath (pp->priv->gc);
+
 	return 0;
 }
 
 static int
 gpp_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
 {
-	/* Nothing.  */
 	return 0;
 }
 
+#if 0
 static Gt1LoadedFont *
 fetch_font (GnomePrintPreviewPrivate *priv, const char *pfb_file, const char *afm_file)
 {
@@ -716,37 +559,23 @@ fetch_font (GnomePrintPreviewPrivate *priv, const char *pfb_file, const char *af
 	}
 	return font;
 }
+#endif
 
 static int
-gpp_show (GnomePrintContext *pc, char const *text)
+gpp_show_ucs4 (GnomePrintContext *pc, guint32 * buf, gint length)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
 	GnomePrintPreviewPrivate *priv = pp->priv;
+	GnomeCanvasGroup *group;
 	GnomeCanvasItem *item;
+	const ArtPoint * cp;
+	const GnomeFont * font;
 	ArtPoint p;
-	Gt1LoadedFont *gt1_font;
-	double transform [6], inverse[6];
-	GraphicContext *g = gc (priv);
+	double transform [6], a[6], inverse[6];
+	GnomeGlyphList * gl;
+	gint i;
 
-	
-	if (g->font == NULL){
-		g_warning ("Show invoked without prior setfont");
-		return -1;
-	}
-
-	if (g->bpath == NULL){
-		g_warning ("You need to moveto first");
-		return -1;
-	}
-
-	gt1_font = fetch_font (priv,
-			       g->font->fontmap_entry->pfb_fn,
-			       g->font->fontmap_entry->afm_fn);
-
-	if (!gt1_font){
-		g_warning ("Could not use the specified font");
-		return -1;
-	}
+	font = gp_gc_get_font (pp->priv->gc);
 
 	/*
 	 * The X and Y positions were computed to be the base
@@ -754,35 +583,88 @@ gpp_show (GnomePrintContext *pc, char const *text)
 	 * Postscript->Canvas translation
 	 */
 	
-	p.x = g->bpath->bpath [g->bpath->n_bpath - 1].x3;
-	p.y = g->bpath->bpath [g->bpath->n_bpath - 1].y3;
+	cp = gp_gc_get_currentpoint (pp->priv->gc);
+
+	memcpy (transform, gp_gc_get_ctm (priv->gc), sizeof (transform));
+	art_affine_scale (a, 1.0, -1.0);
+	art_affine_multiply (transform, a, transform);
 
-	/*
-	 * Flip the angle
-	 */
-	memcpy (transform, g->affine, sizeof (g->affine));
-       
-	transform [1] = -transform [1];
-        transform [2] = -transform [2];
-       
 	art_affine_invert (inverse, transform);
-	art_affine_point (&p, &p, inverse);
+	art_affine_point (&p, cp, inverse);
+	group = (GnomeCanvasGroup *) gp_gc_get_data (pp->priv->gc);
 
-	item = gnome_canvas_item_new (
-		g->group,
+	gl = gnome_glyphlist_from_text_dumb ((GnomeFont *) font, gp_gc_get_rgba (pp->priv->gc),
+					     0.0, 0.0, "");
+	for (i = 0; i < length; i++) {
+		gint glyph;
+		glyph = gnome_font_lookup_default (font, GUINT32_FROM_BE (buf[i]));
+		if (glyph > 0) gnome_glyphlist_glyph (gl, glyph);
+	}
+
+	item = gnome_canvas_item_new (group,
 		gnome_canvas_hacktext_get_type (),
 		"x",           p.x,
 		"y",           p.y,
-		"text",        text,
-		"font",        gt1_font,
-		"size",        g->font->size,
-		"fill_color_rgba", color_rgba (&g->color),
+		"glyphlist",   gl,
 		NULL);
-	
+
+	gnome_glyphlist_unref (gl);
+#if 1
 	gnome_canvas_item_affine_absolute (item, transform);
+#endif
+#if 0
 	priv->current_page++;
 	
 	gc_clear_bpath (gc (priv));
+#endif
+
+	/* fixme: is this correct */
+	gp_gc_newpath (pp->priv->gc);
+
+	return 0;
+}
+
+static int
+gpp_glyphlist (GnomePrintContext *pc, GnomeGlyphList * glyphlist)
+{
+	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
+	GnomePrintPreviewPrivate *priv = pp->priv;
+	GnomeCanvasGroup *group;
+	GnomeCanvasItem *item;
+	const ArtPoint * cp;
+	ArtPoint p;
+	double transform [6], a[6], inverse[6];
+
+
+	/*
+	 * The X and Y positions were computed to be the base
+	 * with the translation already done for the new
+	 * Postscript->Canvas translation
+	 */
+	
+	cp = gp_gc_get_currentpoint (pp->priv->gc);
+
+	memcpy (transform, gp_gc_get_ctm (priv->gc), sizeof (transform));
+	art_affine_scale (a, 1.0, -1.0);
+	art_affine_multiply (transform, a, transform);
+
+	art_affine_invert (inverse, transform);
+	art_affine_point (&p, cp, inverse);
+
+	group = (GnomeCanvasGroup *) gp_gc_get_data (pp->priv->gc);
+
+	item = gnome_canvas_item_new (group,
+		gnome_canvas_hacktext_get_type (),
+		"x",           p.x,
+		"y",           p.y,
+		"glyphlist",        glyphlist,
+		NULL);
+
+	gnome_canvas_item_affine_absolute (item, transform);
+
+	/* fixme: is this correct */
+	gp_gc_newpath (pp->priv->gc);
+
 	return 0;
 }
 
@@ -790,9 +672,9 @@ static int
 gpp_setopacity (GnomePrintContext *pc, double opacity)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	gc(priv)->color.opacity = opacity * 255;
+	gp_gc_set_opacity (pp->priv->gc, opacity);
+
 	return 0;
 }
 
@@ -815,11 +697,15 @@ gpp_destroy (GtkObject *object)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (object);
 	GnomePrintPreviewPrivate *priv = pp->priv;
+
+#if 0
 	GSList *l;
 
 	for (l = priv->context_list; l; l = l->next)
 		gc_free (l->data);
 	g_slist_free (priv->context_list);
+#endif
+	gp_gc_unref (priv->gc);
 
 	if (pp->canvas)
 		gtk_object_unref (GTK_OBJECT (pp->canvas));
@@ -836,6 +722,7 @@ gpp_destroy (GtkObject *object)
 		gtk_object_destroy (GTK_OBJECT (priv->root));
 	
 	g_free (priv);
+
 	GTK_OBJECT_CLASS (print_preview_parent_class)->destroy (object);
 }
 
@@ -865,19 +752,23 @@ gpp_class_init (GnomePrintPreviewClass *class)
 	pc_class->strokepath = gpp_strokepath;
 	pc_class->stroke = gpp_stroke;
 	pc_class->setfont = gpp_setfont;
-	pc_class->show = gpp_show;
+	pc_class->show_ucs4 = gpp_show_ucs4;
 	pc_class->concat = gpp_concat;
+#ifdef SETMATRIX_DEPRECATED
 	pc_class->setmatrix = gpp_setmatrix;
+#endif		
 	pc_class->gsave = gpp_gsave;
 	pc_class->grestore = gpp_grestore;
 	pc_class->clip = gpp_clip;
 	pc_class->eoclip = gpp_eoclip;
 	pc_class->grayimage = gpp_grayimage;
 	pc_class->rgbimage = gpp_rgbimage;
+	pc_class->rgbaimage = gpp_rgbaimage;
 	pc_class->textline = gpp_textline;
 	pc_class->showpage = gpp_showpage;
 	pc_class->beginpage = gpp_beginpage;
 	pc_class->setopacity = gpp_setopacity;
+	pc_class->glyphlist = gpp_glyphlist;
 	
 	pc_class->close = gpp_close;
 }
@@ -886,17 +777,10 @@ static void
 gpp_init (GnomePrintPreview *preview)
 {
 	GnomePrintPreviewPrivate *priv;
-	GraphicContext *gc;
 	
 	priv = preview->priv = g_new0 (GnomePrintPreviewPrivate, 1);
-	gc = g_new0 (GraphicContext, 1);
-	priv->context_list = g_slist_prepend (priv->context_list, gc);
-	
-	set_color (&gc (priv)->color, 0.0, 0.0, 0.0, 0xff);
-	
-	gc (priv)->width = 1.0;
-	gc (priv)->linecap = GDK_CAP_BUTT;
-	gc (priv)->join = GDK_JOIN_MITER;
+
+	priv->gc = gp_gc_new ();
 }
 
 static void
@@ -918,6 +802,9 @@ gnome_print_preview_construct (GnomePrintPreview *preview,
 			       GnomeCanvas *canvas,
 			       const GnomePaper *paper_info)
 {
+	GnomeCanvasGroup * group;
+	double page2root[6];
+
 	g_return_if_fail (preview != NULL);
 	g_return_if_fail (GNOME_IS_PRINT_PREVIEW (preview));
 	g_return_if_fail (canvas != NULL);
@@ -957,16 +844,26 @@ gnome_print_preview_construct (GnomePrintPreview *preview,
 	/*
 	 * Setup base group
 	 */
-	gc (preview->priv)->group = GNOME_CANVAS_GROUP (preview->priv->page);
+
+	group = GNOME_CANVAS_GROUP (preview->priv->page);
+
+	gp_gc_set_data (preview->priv->gc, group);
 
 	/*
 	 * Setup the affines
 	 */
+#if 0
 	art_affine_identity (gc (preview->priv)->affine);
-	art_affine_translate (gc (preview->priv)->base_affine, 0, -gnome_paper_psheight (paper_info));
-	art_affine_flip (gc (preview->priv)->page_affine,
-			 gc (preview->priv)->base_affine,
+	art_affine_translate (page2root, 0, -gnome_paper_psheight (paper_info));
+	art_affine_flip (page2root,
+			 page2root,
 			 FALSE, TRUE);
+#else
+	art_affine_scale (page2root, 1.0, -1.0);
+	page2root[5] = gnome_paper_psheight (paper_info);
+#endif
+
+	gnome_canvas_item_affine_absolute (preview->priv->page, page2root);
 }
 
 /**
