@@ -1,3 +1,11 @@
+/*
+ * gnome-print-ps.c: A Postscript driver for GnomePrint 
+ *
+ * Authors:
+ *   Raph Levien (raph@acm.org)
+ *
+ */
+
 #include "config.h"
 #include <gtk/gtk.h>
 #include <string.h>
@@ -5,8 +13,54 @@
 #include <ctype.h>
 
 #include <libart_lgpl/art_affine.h>
+#include <libgnomeprint/gnome-printer-private.h>
+#include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-ps.h>
 #include <libgnomeprint/gnome-font.h>
+#include <libgnomeprint/gnome-font-private.h> /* for glyph_name at moment */
+
+typedef enum {
+  GNOME_PRINT_PS_TYPE_GENERIC_PS
+} GnomePrintPsType;
+
+struct _GnomePrintPs
+{
+  GnomePrintContext pc;
+
+  /* Number of showpage() preceeding current state.  */
+  int pageno;
+
+  /* the list of fonts that have already been downloaded */
+  int n_fonts, n_fonts_max;
+  char **fonts;
+
+  /* the list of fonts assumed to already exist in the printer */
+  int n_builtins, n_builtins_max;
+  char **builtins;
+
+  /* list of fonts that have been reencoded, and exist int printer */
+  int  n_fonts_reencoded, n_fonts_reencoded_max;
+  char **fonts_reencoded;
+
+  /* current color */
+  double r;
+  double g;
+  double b;
+
+  /* current font */
+  const char *current_font;
+  double current_font_size;
+  const GnomeFont *current_gnome_font;
+
+  /* here follows stuff for printing gnome-text lines */
+  GnomeTextFontHandle cur_font;
+  int cur_size;
+};
+
+struct _GnomePrintPsClass
+{
+  GnomePrintContextClass parent_class;
+};
 
 static void gnome_print_ps_class_init (GnomePrintPsClass *klass);
 
@@ -107,13 +161,6 @@ gnome_print_ps_setrgbcolor (GnomePrintContext *pc,
    * ps->r, etc. are the "last set" color, not the "current color"
    * so you need a color stack to do this optimization.
    */
-#if 0
-  /* if the current color is the color the user wants to set, noop */
-  if (ps->r == r &&
-      ps->g == g &&
-      ps->b == b)
-    return 0;
-#endif
 
   ps->r = r; ps->g = g; ps->b = b;
 
@@ -201,14 +248,14 @@ gnome_print_ps_setopacity (GnomePrintContext *pc, double opacity)
 {
   static gboolean warned = FALSE;
   if (!warned) {
-    g_warning ("Unimplmented setopacity");
+    g_warning ("Unimplemented setopacity");
     warned = TRUE;
   }
   return 0;
 }
 
 static int
-gnome_print_ps_setfont_raw (GnomePrintContext *pc, GnomeFontUnsized *unsized,
+gnome_print_ps_setfont_raw (GnomePrintContext *pc, GnomeFontFace * face,
 			    double size)
 {
   GnomePrintPs *ps;
@@ -220,23 +267,23 @@ gnome_print_ps_setfont_raw (GnomePrintContext *pc, GnomeFontUnsized *unsized,
 
   ps = GNOME_PRINT_PS (pc);
 
-  if (unsized == NULL)
+  if (face == NULL)
     return -1;
 
-  fontname = gnome_font_unsized_get_name (unsized);
+  fontname = gnome_font_face_get_ps_name (face);
   for (i = 0; i < ps->n_builtins; i++)
     if (!strcmp (fontname, ps->builtins[i]))
       break;
 
   if (i == ps->n_builtins)
     {
-      fontname = gnome_font_unsized_get_glyph_name (unsized);
+      fontname = gnome_font_unsized_get_glyph_name (face);
       for (i = 0; i < ps->n_fonts; i++)
 	if (!strcmp (fontname, ps->fonts[i]))
 	  break;
       if (i == ps->n_fonts)
 	{
-	  pfa = gnome_font_unsized_get_pfa (unsized);
+	  pfa = gnome_font_face_get_pfa (face);
 	  if (pfa == NULL)
 	    return -1;
 	  
@@ -267,17 +314,8 @@ gnome_print_ps_setfont_raw (GnomePrintContext *pc, GnomeFontUnsized *unsized,
 	}
     }
 
-  /* NOT always true because of grestore and gsave. Should use
-     a flag to set if grestore has been used **/  
-#if 0
-          if (ps->current_font && strcmp (ps->current_font, fontname) == 0 &&
-              ps->current_font_size == size)
-                  return 0; /* this font is already set */
-#endif
-          
   ps->current_font = fontname;
   ps->current_font_size = size;
-  /* FIXME: what about ->current_gnome_font?  */
 
   return gnome_print_context_fprintf (pc,
 				      "/La-%s findfont %g scalefont setfont\n",
@@ -301,7 +339,7 @@ gnome_print_ps_setfont (GnomePrintContext *pc, GnomeFont *font)
   if (font == NULL)
     return -1;
 
-  fontname = gnome_font_get_name (font);
+  fontname = gnome_font_get_ps_name (font);
   for (i = 0; i < ps->n_builtins; i++)
     if (!strcmp (fontname, ps->builtins[i]))
       break;
@@ -345,15 +383,8 @@ gnome_print_ps_setfont (GnomePrintContext *pc, GnomeFont *font)
 	}
     }
 
-  /* This doesn't consider gsave/grestore... */
-#if 0
-  if (ps->current_font && strcmp (ps->current_font, fontname) == 0 &&
-      ps->current_font_size == font->size)
-    return 0; /* this font is already set */
-#endif
-
   ps->current_font = fontname;
-  ps->current_font_size = font->size;
+  ps->current_font_size = gnome_font_get_size (font);
   ps->current_gnome_font = font;
   /* FIXME: should we ref the font or are we guaranteed to have it around
      for long enough?  */
@@ -364,7 +395,7 @@ gnome_print_ps_setfont (GnomePrintContext *pc, GnomeFont *font)
 
   return gnome_print_context_fprintf (pc,
 				      "/La-%s findfont %g scalefont setfont\n",
-				      fontname, font->size);
+				      fontname, gnome_font_get_size (font));
 
 }
 
@@ -379,42 +410,44 @@ gnome_print_ps_reencode_font (GnomePrintContext *pc, const char *fontname)
 }
 
 static int
-gnome_print_ps_show (GnomePrintContext *pc, char const *text)
+gnome_print_ps_show_ucs4 (GnomePrintContext *pc, guint32 * buf, gint length)
 {
   int i;
   GnomePrintPs *ps;
+  guint32 u;
   ps = GNOME_PRINT_PS (pc);
 
   if (gnome_print_context_fprintf (pc, "(") < 0)
     return -1;
-  for (i = 0; text[i]; i++)
+  for (i = 0; i < length; i++)
     {
-      if (text[i] == '(' || text[i] == ')' || text[i] == '\\') 
+      u = GUINT32_FROM_BE (buf[i]);
+      if (u == '(' || u == ')' || u == '\\') 
 	{
-	  if (gnome_print_context_fprintf (pc, "\\%c", text[i]) < 0)
+	  if (gnome_print_context_fprintf (pc, "\\%c", u) < 0)
 	    return -1;
 	}
-      else if ( text[i] == '-' )
+      else if ( u == '-' )
         {
 	     /* This is a stupid Postscript bug in which all the chars below 127 are
 		the same in the Standard Vector and the ISOLatin encodig BUT this one */
           if (gnome_print_context_fprintf (pc, "\\255") < 0)
 	    return -1;
 	}
-      else if ( text[i] > 31 && text[i] < 127)
+      else if ( u > 31 && u < 127)
         {
-          if (gnome_print_context_fprintf (pc, "%c", text[i]) < 0)
+          if (gnome_print_context_fprintf (pc, "%c", u) < 0)
 	    return -1;
 	}
-      else if ( (unsigned char) text[i] == 153 || (unsigned char) text[i] == 156 ) 
+      else if ( (u == 153) || (u == 156) || (u > 255) ) 
 	{
           /* this chars are .notdef in isoLatin*/ 
 	  if (gnome_print_context_fprintf (pc," ") < 0)
 	    return -1;
 	}
-      else if ( (unsigned char) text[i] > 143 )
+      else if ( (unsigned char) u > 143 )
         {
-          if (gnome_print_context_fprintf (pc, "\\%o", (unsigned char) text[i]) < 0)
+          if (gnome_print_context_fprintf (pc, "\\%o", u) < 0)
 	    return -1;
 	}
       else 
@@ -437,6 +470,7 @@ gnome_print_ps_concat (GnomePrintContext *pc, const double matrix[6])
                                       "%s\n", str);
 }
 
+#ifdef SETMATRIX_DEPRECATED
 static int
 gnome_print_ps_setmatrix (GnomePrintContext *pc, const double matrix[6])
 {
@@ -449,6 +483,7 @@ gnome_print_ps_setmatrix (GnomePrintContext *pc, const double matrix[6])
                                       matrix[4],
                                       matrix[5]);
 }
+#endif
 
 static int
 gnome_print_ps_gsave (GnomePrintContext *pc)
@@ -711,9 +746,11 @@ gnome_print_ps_class_init (GnomePrintPsClass *class)
   pc_class->strokepath = gnome_print_ps_strokepath;
   pc_class->stroke = gnome_print_ps_stroke;
   pc_class->setfont = gnome_print_ps_setfont;
-  pc_class->show = gnome_print_ps_show;
+  pc_class->show_ucs4 = gnome_print_ps_show_ucs4;
   pc_class->concat = gnome_print_ps_concat;
+#ifdef SETMATRIX_DEPRECATED
   pc_class->setmatrix = gnome_print_ps_setmatrix;
+#endif	
   pc_class->gsave = gnome_print_ps_gsave;
   pc_class->grestore = gnome_print_ps_grestore;
   pc_class->clip = gnome_print_ps_clip;
@@ -785,6 +822,7 @@ gnome_print_ps_init (GnomePrintPs *ps)
   ps->builtins = g_new (char *, ps->n_builtins_max);
 
   ps->fonts_reencoded = g_new (char *, ps->n_fonts_reencoded_max);
+
   for (i = 0; i < sizeof(gnome_print_ps_builtins) / sizeof(char *); i++)
     ps->builtins[i] = gnome_print_ps_builtins[i];
 
@@ -822,8 +860,8 @@ gnome_print_ps_new (GnomePrinter *printer)
 				   "%%%% Creator: Gnome Print Version %s\n"
 				   "%%%% DocumentName: %s\n"
 				   "%%%% Author: %s\n"
-				   "%%%%Pages: (atend)\n"
-				   "%%%%EndComments\n\n\n",
+				   "%%%% Pages: (atend)\n"
+				   "%%%% EndComments\n\n\n",
 				   VERSION,
 				   "Document Name Goes Here",
 				   "Author Goes Here") < 0)
