@@ -18,6 +18,38 @@
 #include <libgnomeprint/gnome-printer-profile.h>
 #include <libgnomeprint/gnome-printer-dialog.h>
 
+struct _GnomePrinterWidget {
+	GtkFrame frame;
+	
+	GnomePrinterProfileList *profiles;
+	GnomePrinterProfile *profile;
+	GtkWidget *profile_selector;
+	
+	GtkWidget *label_state;
+	GtkWidget *label_type;
+	GtkWidget *label_location;
+	GtkWidget *label_comment;
+	
+	GtkWidget *r1;
+	GtkWidget *r2;
+	GtkWidget *entry_command;
+	GtkWidget *entry_filename;
+	GtkAccelGroup *accel_group;
+};
+
+struct _GnomePrinterWidgetClass {
+	GtkFrameClass parent_class;
+};
+
+struct _GnomePrinterDialog {
+	GnomeDialog dialog;
+	GnomePrinterWidget *gnome_printer_widget;
+};
+
+struct _GnomePrinterDialogClass {
+	GnomeDialogClass parent_class;
+};
+
 static GnomeDialogClass *dialog_parent_class = NULL;
 static GtkFrameClass    *widget_parent_class = NULL;
 
@@ -64,7 +96,7 @@ set_profile (GnomePrinterWidget *gpw, GnomePrinterProfile *pp)
 	 * Set the entry values
 	 */
 	output = gnome_printer_profile_get_output (pp);
-	if (strncmp (output, "command", 7) == 0){
+	if ((output != NULL) && (strncmp (output, "command", 7) == 0)) {
 		gtk_entry_set_text (GTK_ENTRY (gpw->entry_command), output+8);
 		if (!GTK_TOGGLE_BUTTON (gpw->r1)->active)
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gpw->r1), 1);
@@ -362,6 +394,30 @@ gnome_printer_widget_new (void)
 	return gpw;
 }
 
+void
+gnome_printer_widget_bind_editable_enters (GnomePrinterWidget * gpw, GnomeDialog * dialog)
+{
+	g_return_if_fail (gpw != NULL);
+	g_return_if_fail (GNOME_IS_PRINTER_WIDGET (gpw));
+	g_return_if_fail (dialog != NULL);
+	g_return_if_fail (GNOME_IS_DIALOG (dialog));
+
+	gnome_dialog_editable_enters (dialog, GTK_EDITABLE (gpw->entry_command));
+	gnome_dialog_editable_enters (dialog, GTK_EDITABLE (gpw->entry_filename));
+}
+
+void
+gnome_printer_widget_bind_accel_group (GnomePrinterWidget * gpw, GtkWindow * window)
+{
+	g_return_if_fail (gpw != NULL);
+	g_return_if_fail (GNOME_IS_PRINTER_WIDGET (gpw));
+	g_return_if_fail (window != NULL);
+	g_return_if_fail (GTK_IS_WINDOW (window));
+
+	gtk_window_add_accel_group (window, gpw->accel_group);
+}
+
+
 static void
 gnome_printer_dialog_finalize (GtkObject *object)
 {
