@@ -1,16 +1,14 @@
-#include <config.h>
-#include <gtk/gtk.h>
+/* Must include these two first */
+#include "config.h"
 #include <libgnomeprint/gnome-print-i18n.h>
-#include <libgnomeprint/gnome-printer.h>
 
+#include <gtk/gtk.h>
+#include <libgnomeprint/gnome-printer.h>
+#include <libgnomeprint/gnome-printer-private.h>
 
 static void gnome_printer_class_init (GnomePrinterClass *klass);
-
-static void
-gnome_printer_init (GnomePrinter *printer);
-
-static void
-gnome_printer_finalize (GtkObject *object);
+static void gnome_printer_init (GnomePrinter *printer);
+static void gnome_printer_finalize (GtkObject *object);
 
 static GtkObjectClass *parent_class = NULL;
 
