@@ -1,6 +1,8 @@
 #ifndef __PARSET1_H__
 #define __PARSET1_H__
 
+BEGIN_GNOME_DECLS
+
 /* Header file for parset1.
 
    You have to include these headers additionally:
@@ -31,4 +33,6 @@ gt1_get_kern_pair (Gt1LoadedFont *font, int glyph1, int glyph2);
 char *
 gt1_get_font_name (Gt1LoadedFont *font);
 
+END_GNOME_DECLS
+
 #endif /* __PARSET1_H__ */
